********************************************************************************************************
*                                                                                                      *
*  DragonsLair1.s :  Apple IIgs version of Dragon's Lair game (c) 1983 Don Bluth / (c) 1990 Readysoft. *
*                                                                                                      *
********************************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI      *      Brutal Deluxe Software      *     Dec 2021   *
********************************************************************************************************

*----------   Global Equates : Firmware / Hardware / GS/OS   ---------
KBDREAD             =          $00C000              ; Read Keyboard
CLR80VID            =          $00C00C              ; Disable 80 Column display
SET80VID            =          $00C00D              ; Enable 80 Column display
KBDSTRB             =          $00C010              ; Keyboard Strobe
RDVBLBAR            =          $00C019              ; VBL Status (Bit #7=0 if VBL)
RD80VID             =          $00C01F              ; 80 column status (Bit #7=1 if 80 column)
TBCOLOR             =          $00C022              ; Text / Background color
NEWVIDEO            =          $00C029              ; Apple IIgs Video Mode
VERTCNT             =          $00C02E              ; Video Controler Vertical Counter
CLOCKCTL            =          $00C034              ; Border Color + Clock
SHADOW              =          $00C035              ; Shadowing Register
SOUNDCTL            =          $00C03C              ; Ensoniq 5503 Control Register
SOUNDDATA           =          $00C03D              ; Ensoniq 5503 Data Register
SOUNDADRL           =          $00C03E              ; Ensoniq 5503 Address Low
SOUNDADRH           =          $00C03F              ; Ensoniq 5503 Address High
BUTN0               =          $00C061              ; Open Apple Key = Joystick Button 0
TEXTPAGE1           =          $E00400              ; Text Page #1 $0400-$07FF
GSOS                =          $E100A8              ; GS/OS Entry Point
CTRLPNLSNDVOL       =          $E100CA              ; Control Panel Sound Volume

*----------   Merlin 16+ Directives   ---------

                    mx         %00                  ; Assemble in 16 bit

                    rel                             ; Build a relocated S16 file
                    dsk        Main.l               ; Name of the Segment

                    use        4/Locator.Macs       ; Macro Definition Files
                    use        4/Mem.Macs
                    use        4/Misc.Macs
                    use        4/Util.Macs
                    use        4/Sound.Macs

*----------   Merlin 16+ Macros   ---------

LogLine             MAC                             ; Define the LogLine Macro
                    LDA        ]1
                    JSR        AddLogLine
                    <<<                             ; End of Macro

*----------   External Labels from DragonsLair1.Bitmap.s Segment  ---------
MissionIsOver       EXT                             ; Bitmap
PressFireButton     EXT
FinalScore          EXT
Aplha_Action        EXT
Aplha_Event         EXT
Aplha_Up            EXT
Aplha_Down          EXT
Aplha_Left          EXT
Aplha_Right         EXT
Aplha_Bullet        EXT

NumberTable         EXT                             ; Table
DiskTable           EXT
Alpha_Char          EXT
Alpha_Tab           EXT
Aplha_Diskette      EXT

EntryPointBitmap    EXT                             ; Function
CopyOneBitmap       EXT
CopyTransBitmap     EXT
RemapBitmapColor    EXT
DisplayHintBitmap   EXT
DisplayDiskBitmap   EXT

**********************************************
*  Main() :  Entry point of the Application. *
********************************************** 

Main                PHK                             ; Data Bank Register = Program Bank Register
                    PLB
                    SEP        #$20
                    PHK                             ; Save Current Bank
                    PLA
                    STA        CurrentBank          ; 00XX 

                    STA        FIN_1+3              ; Define Current Bank as the Source for FadeIn Picture
                    STA        FIN_5+3
                    STA        FIN_8+3
                    STA        FIN_11+3                    
                    
M_Bitmap            JSL        EntryPointBitmap     ; Used to get Bitmap Segment Bank
                    LDA        M_Bitmap+3           ; Bitmap Segment Bank
                    STA        IDS_BitmapBank_1+3
                    STA        IDS_BitmapBank_2+3
                    
                    CLC                             ; 16 bit
                    XCE
                    REP        #$30

*-----   Program Initialization   ---------

M_Init              JSR        ToolInitialize       ; Init Tools + Compact Memory + Ask for Shadowing
                    JSR        SwitchTwilightOff    ; Switch Off Twilight II (if there)
                    JSR        BackupEnvironment    ; Backup environment (colors...)

*-----   Allocate Memory + Check Memory Prerequisites

M_Alloc             JSR        AllocProgramMemory   ; Memory Allocation (Mem + Float)
                    BCC        M_Startup
                    JMP        Error_Memory

*-----   Startup Screen using Text Page #1 + 40 column Mode ----------

M_Startup           JSR        StartupScreen        ; Display Text Startup Screen + Load 3 Data Files
                    BEQ        M_HeartBeat
                    JMP        Error_LoadFile       ; A = DataFile ID
                    
*-----   Start HeartBeat 1/60s Interruptions   ---------

M_HeartBeat         SEI                             ; Enable HeartBeat
                    PushLong   #HBEntryPoint
                    _SetHeartBeat
                    CLI
                    INC        M_End+1              ; HeartBeat is set
                            
*-----   Boot Sequence : Load Minimum File Set + Display Screens & Scenes   ------

M_BootSequence      JSR        BootSequence         ; Display Title + Credit + Attract (Scenes 03 & 0B)
                    BCC        M_BootAction
                    JMP        Error_LoadFile       ; A = DataFile ID
*--                    
M_BootAction        CMP        #$0051               ; Q = Quit
                    BEQ        M_End
                    CMP        #$004C               ; L = Load Game + Play Game
                    BEQ        M_LoadGame
                    CMP        #$0045               ; Esc : Escape
                    BEQ        M_AttractSequence
                    CMP        #$0030               ; 0 : Play New Game
                    BEQ        M_GameInit      
                    BRA        M_AttractSequence    ; Next Sequence => Attract 
                    
*-----   Attract Sequence : Show Title & Credit screens + Attract Scene   --------

M_AttractSequence   JSR        AttractSequence      ; Display Title + Credit + Scenes 03 & 0B
                    BEQ        M_AttractSequence
                    CMP        #$0051               ; Q = Quit to Finder
                    BEQ        M_End
                    CMP        #$004C               ; L = Load Game + Play Game
                    BEQ        M_LoadGame
                    CMP        #$0045               ; Esc : Escape
                    BEQ        M_AttractSequence
                    CMP        #$0030               ; 0 : Play New Game
                    BEQ        M_GameInit
                    BRA        M_AttractSequence    ; Loop on Attract Sequence

*-----   Game Sequence : Show Difficulty & Score/Level screens + Scenes 6+ -> N   --------

M_LoadGame          JSR        LoadGame             ; Load Saved Game data (if exist) => A = Scene #
                    BRA        M_GameSequence
*--
M_GameInit          JSR        ResetScoreLife       ; Init Level + Score + Life
                    LDA        #$000B               ; Starts at Scene #0B

M_GameSequence      JSR        GameSequence         ; Play Scenes 6+ -> N
                    CMP        #$FFFF               ; Fatal Error
                    BEQ        M_End
                    CMP        #$0051               ; Q : Quit
                    BEQ        M_End
                    CMP        #$004C               ; L : Load Game + Play Game
                    BEQ        M_LoadGame
                    CMP        #$0045               ; Esc : Escape
                    BEQ        M_AttractSequence
*--
                    BRA        M_AttractSequence

*-----   End Of Program   ---------

M_End               LDA        #$0000               ; Set to 1 if HeartBeat previously registered
                    BEQ        M_End_1
*--                    
                    SEI                             ; Delete HeartBeat
                    PushLong   #HBEntryPoint
                    _DelHeartBeat
                    CLI
*--              
M_End_1             JSR        ClearTextPage1       ; Clear Page Text #1
                    JSR        RestoreEnvironment   ; Restore environment (colors...)
                    JSR        SwitchTwilightOn     ; Switch On Twilight II
                    JSR        ToolTerminate        ; End up Tools
                    JMP        ExitToLauncher       ; Quit to the Launcher


*********************************************************************************
*  Error_XXX() :  Error Handling Routine (Memory allocation, File access, ...). *
*********************************************************************************
Error_Memory        LDX        #ERR_LineMemory1     ; Memory Allocation Error
                    LDY        #ERR_LineMemory2
                    BRA        ErrorHandling
*----
Error_LoadFile      CLC                             ; A = File ID from Disk #1 (001,002,003..)
                    ADC        #$AEB0               ; ".0"
                    STA        ERR_LineLoadFile2+28
                    LDX        #ERR_LineLoadFile1   ; Can't Load Data File
                    LDY        #ERR_LineLoadFile2
                    BRA        ErrorHandling
*----                    
ErrorHandling       STX        Error_TabLine+2      ; Install Error Lines
                    STY        Error_TabLine+4
                    JSR        Set40ColTextPage1    ; Display Text Page #1 / 40 Column                    
                    JSR        ClearTextPage1       ; Clear Page Text #1
                    LDA        #Error_TabLine
                    JSR        BuildTextScreen      ; Display Text Error message
*--
                    JSR        WaitForKey           ; Wait for Key Press
*--
                    JMP        M_End                ; End Program

*--------------   Error Screen Variables   -------------
Error_TabLine       DA         ERR_Line1,ERR_Line2,ERR_Line2,ERR_Line3
                    HEX        0000

ERR_Line1           HEX        0400,0000            ; Y=4, X=0
                    ASC        "      A Fatal Error Has Occurred !      ",00
ERR_Line2           HEX        1000,0000            ; Y=16, X=0
                    ASC        "                                        ",00
ERR_Line3           HEX        1600,0000            ; Y=22, X=0
                    ASC        "       - Press Any Key to Exit -        ",00
*--
ERR_LineMemory1     HEX        0F00,0000            ; Y=15, X=0
                    ASC        "  Error, Can't allocate enough memory.  ",00
ERR_LineMemory2     HEX        1000,0000            ; Y=16, X=0
                    ASC        "This program requires 1.1 MB of free RAM",00
*--
ERR_LineLoadFile1   HEX        0A00,0000            ; Y=8, X=0
                    ASC        " Error, Can't load Data File from Disk :",00
ERR_LineLoadFile2   HEX        0D00,0000            ; Y=10, X=0
                    ASC        "          1/Disk1/Data00X.bin           ",00


************************************************************************************
*  AllocProgramMemory() :  Allocate Memory for the Program (OUT:Carry=SetOnError). *
************************************************************************************
AllocProgramMemory  LDA        REQUIRED_FLOAT_BANK  ; Memory Allocation for Float
                    LDX        #BankDataFloat
                    JSR        MemoryAllocation
                    CMP        REQUIRED_FLOAT_BANK
                    BMI        APM_Error
*----
                    LDA        MAXIMUM_RAM_BANK     ; Memory Allocation for Mem
                    LDX        #TableAllocatedBank
                    JSR        MemoryAllocation
                    STA        NbAllocatedBank
                    CMP        MINIMUM_RAM_BANK
                    BMI        APM_Error
*----
APM_End             CLC                             ; OK
                    RTS  
*--
APM_Error           SEC                             ; Error
                    RTS  

*--------  AllocProgramMemory variables  --------
CurrentBank         HEX        0000                 ; Program Current Bank 00XX
MINIMUM_RAM_BANK    =          #$0009               ; 9 : Attract
REQUIRED_FLOAT_BANK =          #$0008               ; 8 floating banks for Scene #17
MAXIMUM_RAM_BANK    =          #$0060               ; 96 banks    
NbAllocatedBank     HEX        0000                 ; Number allocated Ram Bank
*--
TableAllocatedBank  
BankData001         HEX        0000                 ; Data001 = Dragon's Lair Title Screen / Insert Disk Screen / Credit Screen / Attract Scene #03
BankData002         HEX        0000                 ; Data002 = Attract Scene #03
BankData003         HEX        0000                 ; Data003 = Scene #0B 
BankData004         HEX        0000                 ; Data004 = Scene #0B
BankData005         HEX        0000                 ; Data005 = Scene #0B
BankData006         HEX        0000                 ; Data006 = Scene #0B
BankData007         HEX        0000                 ; Data007 = Scene #0B
BankData008         HEX        0000                 ; Data008 = Scene #0B
BankData009         HEX        0000                 ; Data009 = Score + Lives Screen / Scene #06
BankDataXXX         DS         2*87                 ; Space for all 87 banks pointers (9+87 = 96 required banks) 
*--
BankDataFloat       DS         2*8                  ; Space for 8 floating banks
*----
BankData001ID       HEX        0100                 ; Data0ID stored in allocated BankData0ID 
BankData002ID       HEX        0200 
BankData003ID       HEX        0300
BankData004ID       HEX        0400
BankData005ID       HEX        0500
BankData006ID       HEX        0600
BankData007ID       HEX        0700
BankData008ID       HEX        0800
BankData009ID       HEX        0900
BankDataXXXID       DS         2*87
*--
BankDataFloatID     DS         2*8                  ; DataID stored in floating BankData 


********************************************************************************************************************************
*  BootSequence() :  Load Minimum Data files + Display Title + Credit + Attract (OUT:A=ActionCode|Error,OUT:Carry=SetOnError). *
********************************************************************************************************************************
BootSequence        JSR        ClearKDBStrobe       ; Clear Keyboard
*--
                    LDA        #$0001               ; Scene 01 / Type 00 => Title Screen
                    LDX        #$0000               ; Type
                    LDY        #$0000               ; Frame      
                    JSR        GetSceneOffset       ; => X=Offset, Y=Length                  
                    LDA        BankData001          ; A=XX00
                    ORA        CurrentBank          ;   00YY
                    JSR        UnpackLZ4            ; A = BankSrc,BankDst / X = 1st Packed Byte offset / Y = Pack Data Size
*--
                    SEP        #$20                 ;
                    LDA        #$00                 ; Shadowing ON
                    STAL       SHADOW
                    REP        #$30
*--
                    LDX        #$1FFE               ; RAZ Graphic Page
                    LDA        #$000
BS_RAZ_GP           STAL       $012000,X
                    STAL       $014000,X
                    STAL       $016000,X
                    STAL       $018000,X
                    DEX
                    DEX
                    BPL        BS_RAZ_GP
*--
                    SEP        #$20                          
                    LDA        #$C1                 ; Display SHR + Linarize Graphic Page
                    STAL       NEWVIDEO
                    REP        #$30
*--
                    JSR        ResetTimerSec        ; Start Timer

*----   Display Title Screen   ----
BS_Title            LDA        #BUFFER32KB          ; FadeIn to Title Screen
                    LDY        #$0000               ; Copy Pixel + SCB from Src Picture
                    JSR        FadeIN
*--
BS_LoadDataFile004  LDA        #$0004               ; Load Data004.bin        
                    LDX        BankData004
                    LDY        #$0000               ; Don't ask for Floppy (should be on boot drive)
                    JSR        LoadDataFile  
                    BCC        BS_LoadDataFile005
                    LDA        #$0004
                    JMP        BS_EndError
BS_LoadDataFile005  LDA        #$0005               ; Load Data005.bin        
                    LDX        BankData005
                    LDY        #$0000
                    JSR        LoadDataFile  
                    BCC        BS_LoadDataFile006        
                    LDA        #$0005
                    JMP        BS_EndError
BS_LoadDataFile006  LDA        #$0006               ; Load Data006.bin        
                    LDX        BankData006
                    LDY        #$0000
                    JSR        LoadDataFile  
                    BCC        BS_Title_3Sec        
                    LDA        #$0006
                    JMP        BS_EndError
*----
BS_Title_3Sec       JSR        CheckUserAction      ; Check for Key Pressed
                    BEQ        BS_Title_3Sec_2
                    CMP        #$0051               ; Q = Quit to Finder
                    BNE        BS_Title_3Sec_1
                    JMP        BS_End
BS_Title_3Sec_1     CMP        #$0030               ; Action = Next Screen
                    BEQ        BS_Title_EOS
BS_Title_3Sec_2     JSR        GetTimerSec          ; We need to wait at least 3 sec
                    CMP        #$0003
                    BMI        BS_Title_3Sec
*-- 
BS_Title_EOS        JSR        StopSound            ; Make Sure Sound is not playing           
           
*----   Display Credit Screen   ----
BS_Credit           JSR        FadeOut              ; FadeOut to Black
*--
                    LDA        #$0005               ; Scene 05 / Type 00 => Credits Screen
                    LDX        #$0000               ; Type
                    LDY        #$0000               ; Frame      
                    JSR        GetSceneOffset       ; => X=Offset, Y=Length                  
                    LDA        BankData001          ; A=XX00
                    ORA        CurrentBank          ;   00YY
                    JSR        UnpackLZ4            ; A = BankSrc,BankDst / X = 1st Packed Byte offset / Y = Pack Data Size
                    CMP        #$8000               ; => Return in A the length of unpacked Data
*--                    
                    LDA        #BUFFER32KB          ; FadeIn to Credit Screen
                    LDY        #$0000               ; Copy Pixel + SCB from Src Picture
                    JSR        FadeIN
*--
                    JSR        ResetTimerSec        ; Start Timer
*--
BS_LoadDataFile007  LDA        #$0007               ; Load Data007.bin
                    LDX        BankData007
                    LDY        #$0000
                    JSR        LoadDataFile
                    BCC        BS_LoadDataFile008
                    LDA        #$0007
                    JMP        BS_EndError                      
BS_LoadDataFile008  LDA        #$0008               ; Load Data008.bin
                    LDX        BankData008
                    LDY        #$0000
                    JSR        LoadDataFile
                    BCC        BS_LoadDataFile009
                    LDA        #$0008
                    JMP        BS_EndError                    
BS_LoadDataFile009  LDA        #$0009               ; Load Data009.bin
                    LDX        BankData009
                    LDY        #$0000
                    JSR        LoadDataFile
                    BCC        BS_Credit_3Sec
                    LDA        #$0009
                    JMP        BS_EndError                    
*--
BS_Credit_3Sec      JSR        CheckUserAction      ; Check for Key Pressed
                    BEQ        BS_Credit_3Sec_1   
                    CMP        #$0051               ; Q = Quit to Finder
                    BEQ        BS_End
                    CMP        #$0045               ; Esc = Go to Title Screen
                    BEQ        BS_End
                    CMP        #$0030               ; Action = Start Game
                    BEQ        BS_End
                    CMP        #$004C               ; L = Load Game
                    BEQ        BS_End
BS_Credit_3Sec_1    JSR        GetTimerSec          ; We need to wait at least 3 sec
                    CMP        #$0003
                    BMI        BS_Credit_3Sec

*----   Display Attract Scene #03   ----
BS_Attract_3        LDA        #$0003               ; Play Scene #03
                    LDX        #$0001               ; Is Attract Mode
                    JSR        PlayScene
                    BNE        BS_End                              

*----   Display Attract Scene #0B   ----
BS_Attract_B        LDA        #$000B               ; Play Scene #0B
                    LDX        #$0001               ; Is Attract Mode
                    JSR        PlayScene
*--                    
BS_End              PHA                             ; No Error
                    JSR        StopSound            ; Stop Sound if any
                    PLA
                    CLC                             
                    RTS
*--
BS_EndError         PHA                             ; Fatal Error : Can't load Data File
                    JSR        StopSound            ; Stop Sound if any
                    PLA
                    SEC
                    RTS


*********************************************************************************************
*  AttractSequence() :  Display Title & Credit screens + Attract scenes (OUT:A=ActionCode). *
*********************************************************************************************
AttractSequence     JSR        ClearKDBStrobe       ; Clear Keyboard
*--
                    JSR        FadeOut              ; FadeOut to Black
*----
AS_Title            LDA        #$0001               ; Scene 01 / Type 00 => Title Screen
                    LDX        #$0000               ; Type
                    LDY        #$0000               ; Frame      
                    JSR        GetSceneOffset       ; => X=Offset, Y=Length                  
                    LDA        BankData001          ; A=XX00
                    ORA        CurrentBank          ;   00YY
                    JSR        UnpackLZ4            ; A = BankSrc,BankDst / X = 1st Packed Byte offset / Y = Pack Data Size
*--
                    JSR        ResetTimerSec        ; Start Timer
*--
                    LDA        #BUFFER32KB          ; FadeIn to Title Screen
                    LDY        #$0000               ; Copy Pixel + SCB from Src Picture
                    JSR        FadeIN
*--
                    JSR        LoadNextDataFile     ; Load one extra Data File
                    JSR        LoadNextDataFile     ; Load one extra Data File
*--
AS_Title_3Sec       JSR        CheckUserAction      ; Check for Key Pressed
                    BEQ        AS_Title_3Sec_1
                    CMP        #$0051               ; Q = Quit to Finder
                    BEQ        AS_Exit
                    CMP        #$0045               ; Esc = Go to Title Screen
                    BNE        AS_Exit                    
                    CMP        #$004C               ; L = Load Level
                    BEQ        AS_Exit
                    CMP        #$0030               ; Action = Start Game
                    BEQ        AS_Credit
AS_Title_3Sec_1     JSR        GetTimerSec          ; We need to wait at least 3 sec
                    CMP        #$0003
                    BMI        AS_Title_3Sec
*----
AS_Credit           JSR        FadeOut              ; FadeOut to Black
                    LDA        #$0005               ; Scene 05 / Type 00 => Credits Screen
                    LDX        #$0000               ; Type
                    LDY        #$0000               ; Frame      
                    JSR        GetSceneOffset       ; => X=Offset, Y=Length                  
                    LDA        BankData001          ; A=XX00
                    ORA        CurrentBank          ;   00YY
                    JSR        UnpackLZ4            ; A = BankSrc,BankDst / X = 1st Packed Byte offset / Y = Pack Data Size
*--                    
                    LDA        #BUFFER32KB          ; FadeIn to Credit Screen
                    LDY        #$0000               ; Copy Pixel + SCB from Src Picture
                    JSR        FadeIN
*--
                    JSR        ResetTimerSec        ; Start Timer
*--
                    JSR        LoadNextDataFile     ; Load one extra Data File
                    JSR        LoadNextDataFile     ; Load one extra Data File
*--
AS_Credit_3Sec      JSR        CheckUserAction      ; Check for Key Pressed
                    BEQ        AS_Credit_3Sec_1   
                    CMP        #$0051               ; Q = Quit to Finder
                    BEQ        AS_Exit
                    CMP        #$004C               ; L = Load Level
                    BEQ        AS_Exit                    
                    CMP        #$0045               ; Esc = Go to Title Screen
                    BEQ        AS_Exit                    
                    CMP        #$0030               ; Action = Start Game
                    BEQ        AS_Attract_3
AS_Credit_3Sec_1    JSR        GetTimerSec          ; We need to wait at least 3 sec
                    CMP        #$0003
                    BMI        AS_Credit_3Sec
*----
AS_Attract_3        LDA        #$0003               ; Play Scene #03
                    LDX        #$0001               ; Is Attract Mode
                    JSR        PlayScene
                    BNE        AS_Exit                             
*----
AS_Attract_B        LDA        #$000B               ; Play Scene #0B
                    LDX        #$0001               ; Is Attract Mode
                    JSR        PlayScene
                    BNE        AS_Exit
*--
AS_End              LDA        #$0000               ; Action Code = Continue
AS_Exit             PHA                             ; Stop Sound
                    JSR        StopSound
                    PLA
                    RTS


*******************************************************************************************************
*  GameSequence() :  Display Difficulty & ScoreLife screens + Play Scene (IN:A=SceneID,OUT:A=Action). *
*******************************************************************************************************
GameSequence        STA        GS_PlayScene_Init_1+1 ; Starting Scene ID
                    JSR        ClearKDBStrobe       ; Clear Keyboard
*-----
                    LDA        #$0002               ; Default Choice : Easy
                    JSR        DifficultyScreen     ; Display Choose Difficulty Screen
*-----      
GS_ScoreLife        JSR        ResetTimerSec        ; Start Timer
*--
                    LDA        #$000                ; Display Score/Life Screen
                    JSR        ScoreLifeScreen      
                    JSR        LoadNextDataFile     ; Load one extra Data File
                    JSR        LoadNextDataFile     ; Load one extra Data File
*--
GS_ScoreLife_3Sec   JSR        CheckUserAction      ; Check for Key Pressed
                    BEQ        GS_ScoreLife_3Sec_1      
                    CMP        #$0045               ; Esc = Escape to Title screen
                    BEQ        GS_ScoreLife_End
                    CMP        #$004C               ; L = Load Game
                    BEQ        GS_ScoreLife_End
                    CMP        #$0051               ; Q = Quit to Finder
                    BEQ        GS_ScoreLife_End
                    CMP        #$0030               ; Action = Next Screen
                    BEQ        GS_PlayScene                    
GS_ScoreLife_3Sec_1 JSR        GetTimerSec          ; We need to wait at least 3 sec
                    CMP        #$0003
                    BMI        GS_ScoreLife_3Sec
                    BRA        GS_PlayScene
GS_ScoreLife_End    JMP        GS_End
*--
GS_PlayScene        JSR        FadeOut              ; FadeOut to Black
*-----
GS_PlayScene_Init   LDX        #$0000               ; Seach for first Scene ID
GS_PlayScene_Init_0 LDA        GS_PlayScene_Tab,X   ; Scene ID
GS_PlayScene_Init_1 CMP        #$AAAA               ; First Scene ID to play
                    BEQ        GS_PlayScene_Init_2
                    INX
                    INX
                    BRA        GS_PlayScene_Init_0
GS_PlayScene_Init_2 STX        GS_PlayScene_1+1
*-----
GS_PlayScene_1      LDX        #$AAAA               ; Loop over all Game Scene
                    LDA        GS_PlayScene_Tab,X   ; Scene ID
                    STA        GS_PlayScene_2+1
                    CMP        #$003C               ; Last Scene => The End Screen
                    BEQ        GS_YouWin
*--
                    JSR        LoadSceneDataFile    ; Load all Data File required for this Scene
                    CMP        #$FFFF               ; Fatal Error => Exit
                    BNE        GS_PlayScene_2
                    JMP        GS_End
*--                 
GS_PlayScene_2      LDA        #$AAAA               ; Play Game : Scene ID
                    LDX        #$0000               ; Game Mode
                    JSR        PlayScene
                    BEQ        GS_PlayScene_3       ; We can continue to next Scene
                    CMP        #$FFFF
                    BNE        GS_End               ; Something has happended => Exit with inner Action Code
*--
GS_PlaySceneDeath   JSR        GetScoreLife
                    TXA                             ; X = Life|Score1
                    AND        #$FF00
                    BNE        GS_PlayScene_1       ; Restart same Scene
                    BRA        GS_Exit              ; Out of Life => Exit with Action Code 0
*--                    
GS_PlayScene_3      INC        GS_PlayScene_1+1
                    INC        GS_PlayScene_1+1
                    BRA        GS_PlayScene_1
*-----
GS_YouWin           JSR        FadeOut              ; FadeOut to Black
*--
                    LDA        #$003C               ; Scene 3C = The End Screen
                    JSR        LoadSceneDataFile    ; Load all Data File required for this Scene
                    LDA        #$003C
                    LDX        #$0000               ; Type
                    LDY        #$0000               ; Frame      
                    JSR        GetSceneOffset       ; => X=Offset, Y=Length
                    PHX
                    LDA        #$003C                  
                    JSR        GetPictureBank       ; => A=XX00
                    ORA        CurrentBank          ;      00YY
                    PLX
                    JSR        UnpackLZ4            ; A = BankSrc,BankDst / X = 1st Packed Byte offset / Y = Pack Data Size
*--                    
                    LDA        #BUFFER32KB          ; FadeIn to YouWin Screen
                    LDY        #$0000               ; Copy Pixel + SCB from Src Picture
                    JSR        FadeIN
                    JSR        ResetTimerSec        ; Start Timer
GS_YouWin_3Sec      JSR        HasKeyPressed        ; Score Screen if Key pressed
                    BCS        GS_Score      
                    JSR        GetTimerSec          ; We need to wait at least 3 sec
                    CMP        #$0003
                    BMI        GS_YouWin_3Sec
*--
GS_Score            JSR        GetDifficulty        ; Do not display Score Screen if Moovie
                    CMP        #$0001               ; None
                    BEQ        GS_Exit
                    LDA        #$0001               ; Display Final Score/Life Screen
                    JSR        ScoreLifeScreen
                    JSR        ResetTimerSec        ; Start Timer
GS_Score_3Sec       JSR        HasKeyPressed        ; Exit if Key pressed
                    BCS        GS_Exit
                    JSR        GetTimerSec          ; We need to wait at least 3 sec
                    CMP        #$0003
                    BMI        GS_Score_3Sec
*--
GS_Exit             LDA        #$0000               ; Action Code = Continue
GS_End              RTS

*----------   GameSequence Variables   ---------
GS_PlayScene_Tab    HEX        0B00,0600,0700,0800  ; Starts with Scene #0B
                    HEX        0900,0A00,0C00,0D00
                    HEX        0E00,0F00,1000,1100
                    HEX        1200,1300,1300,1300  ; Repeats Scene #13 3 times
                    HEX        1400,1500,1600,1700
                    HEX        1800,1900,1A00,1B00
                    HEX        1C00,1D00,1E00,1F00
                    HEX        2000,2100,2200,2300  ; Scene #24 is missing
                    HEX        2500,2600,2700,2800
                    HEX        2900,2A00,2B00,2C00               
                    HEX        2D00,2E00,2F00,3000
                    HEX        3100,3200,3300,3400
                    HEX        3C00                 ; Scene #3C is the last Scene (The End Screen)


****************************************************************************************************************************
*  LoadDataFile() :  Load a Disk*/Data***.bin file into memory (IN:A=File#,IN:X=Bank,IN:Y=TryOnFloppy,OUT:Carry=1OnError). *
****************************************************************************************************************************
LoadDataFile        STX        LDF_TargetBank       ; X = Bank XX/00
                    STY        LDF_TryOnFloppy+1    ; 0001 if Try to load from Floppy
*--
LDF_BuildPath       TAX                             ; A = File # 001-101
                    LDA        #LDF_FileData_HDrive
                    JSR        BuildFileDataPath    ; Build HardDrive File Path
*--
LDF_LoadFileHDRive  LDX        #LDF_FileData_HDrive ; X = File Path 
                    LDA        LDF_TargetBank       ; A = Bank XX/00
                    JSR        LoadFile             ; Set Carry on Error
                    BCS        LDF_TryOnFloppy
                    JMP        LDF_End
*--
LDF_TryOnFloppy     LDA        #$AAAA               ; Can't load file, Ask Floppy ?
                    BNE        LDF_BuildFloppyPath
                    JMP        LDF_End             
*--
LDF_BuildFloppyPath SEP        #$20                 ; Build Floppy File Path
                    LDA        LDF_FileData_HDrive+8 ; Disk #
                    STA        LDF_FileData_Floppy+17
                    STA        LDF_FileData_Floppy+23
                    STA        LDF_AskForFloppy+1
                    REP        #$20
                    LDA        LDF_FileData_HDrive+14 ; File #
                    STA        LDF_FileData_Floppy+29
                    LDA        LDF_FileData_HDrive+15
                    STA        LDF_FileData_Floppy+30
*--
LDF_LoadFileFloppy  LDX        #LDF_FileData_Floppy ; X = File Path 
                    LDA        LDF_TargetBank       ; A = Bank XX/00
                    JSR        LoadFile             ; Set Carry on Error
                    BCS        LDF_AskForFloppy
                    JMP        LDF_End       
*--
LDF_AskForFloppy    LDA        #$00AA               ; A = Disk Number 1-7
                    SEC
                    SBC        #$0030               ; '1' to 1
                    JSR        InsertDiskScreen
*--
LDF_LoadFileRetry   LDX        #LDF_FileData_Floppy ; X = File Path 
                    LDA        LDF_TargetBank       ; A = Bank XX/00
                    JSR        LoadFile             ; Set Carry on Error
                    BCC        LDF_End       
*----
                    LDX        #$001E               ; Display Error Message
LDF_InstallPalette  LDA        LDF_Palette,X
                    STAL       $E19E20,X
                    DEX
                    DEX
                    BPL        LDF_InstallPalette
*--
                    LDX        #$001E
                    LDA        #$0101               ; Palette #1
LDF_InstallSCB      STAL       $E19D54,X
                    DEX
                    DEX
                    BPL        LDF_InstallSCB
*--
                    LDX        #$13FE               ; Clear 32 lines
                    LDA        #$0000
LDF_Cleanup         STAL       $E15480,X
                    DEX
                    DEX
                    BPL        LDF_Cleanup
*--
                    LDX        #$0008               ; Copy File Name
LDF_Message         LDA        LDF_FileData_HDrive+8,X
                    STA        LDF_ErrorMessage_1+20,X
                    DEX
                    DEX
                    BPL        LDF_Message
*--
                    LDX        #LDF_ErrorMessage_1  ; Can't Load File
                    LDY        #$570D
                    JSR        DisplayString  
                    LDX        #LDF_ErrorMessage_2  ; Retry or Cancel
                    LDY        #$5FEA
                    JSR        DisplayString  
*--                    
LDF_WaitForKey      JSR        WaitForKey           ; Wait for a Key
                    CMP        #$0052               ; R
                    BEQ        LDF_Retry
                    CMP        #$0072               ; r
                    BEQ        LDF_Retry
                    CMP        #$0043               ; C
                    BEQ        LDF_Cancel
                    CMP        #$0063               ; c
                    BEQ        LDF_Cancel
                    BRA        LDF_WaitForKey 
*--
LDF_Retry           LDX        #LDF_FileData_Floppy ; X = File Path 
                    LDA        LDF_TargetBank       ; A = Bank XX/00
                    JSR        LoadFile             ; Set Carry on Error
                    BCC        LDF_End
                    BCS        LDF_WaitForKey
*--
LDF_Cancel          SEC                             ; Error
*--
LDF_End             RTS                             ; OK                           

*----------   LoadDataFile Variables   ---------
LDF_TargetBank      HEX        0000
LDF_FileData_HDrive strl       '1/DiskX/DataXXX.bin'
LDF_FileData_Floppy strl       '/DragonsLair1.DX/DiskX/DataXXX.bin'
LDF_ErrorMessage_1  ASC        'Can"t load File DiskX/DataXXX.bin',00
LDF_ErrorMessage_2  ASC        '[R]etry or [C]ancel',00
LDF_Palette         HEX        4A00,FF0F,0000,800F,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000


******************************************************************
*  LoadNextDataFile() :  Load next available Data file (if any). *
******************************************************************
LoadNextDataFile    LDY        #$0000               ; Search for a free entry in the table
                    LDX        #$0000
LNDF_Search         LDA        BankData001ID,X
                    BEQ        LNDF_Load            ; Found
                    INX
                    INX
                    INY
                    CPY        NbAllocatedBank
                    BNE        LNDF_Search
                    BRA        LNDF_End             ; No more file to load
*--
LNDF_Load           STX        LSDF_Store_2+1
                    LDA        BankData001,X        ; XX00
                    PHA                               
                    TYA                             ; Load DataYYY.bin
                    INC                             ; A = DateFile ID
                    JSR        Hex2HexDec           ; Convert $10 as $16
                    STA        LSDF_Store_1+1
                    PLX                             ; X = Mem Bank where to load the File
                    LDY        #$0000               ; Don't ask for Floppy   
                    JSR        LoadDataFile
                    BCS        LNDF_End             ; Error, Can't load file
*--
LSDF_Store_1        LDA        #$AAAA               ; DataFile ID (Decimal)
LSDF_Store_2        LDX        #$AAAA               ; Location 
                    STA        BankData001ID,X         
*--
LNDF_End            RTS


***************************************************************************************************
*  LoadSceneDataFile() :  Load all DataXXX.bin files for this Scene (IN:A=SceneID,OUT:A=#Loaded). *
***************************************************************************************************
LoadSceneDataFile   STA        LSDF_Address_2+1     ; Scene ID
                    STZ        LSDF_DataFileNb
                    STZ        LSDF_DataFileLoaded
*--------
                    LDX        #$0000
LSDF_Address_1      LDA        LSDF_SceneID_Tab,X  ; Search for Scene #ID          
LSDF_Address_2      CMP        #$AAAA                    
                    BEQ        LSDF_Address_3
                    INX
                    INX
                    BRA        LSDF_Address_1
LSDF_Address_3      LDA        LSDF_SceneID_Addr,X
                    STA        LSDF_DataFile+1
*--------
                    LDX        #$0000               ; Collect all DataFile ID for this Scene
LSDF_DataFile       LDA        $AAAA,X
                    BEQ        LSDF_Load
                    STA        LSDF_DataFileTab,X
                    INC        LSDF_DataFileNb
                    INX
                    INX
                    BRA        LSDF_DataFile
*--------
LSDF_Load           LDA        LSDF_DataFileNb      ; Allocate/Load all required DataFile
                    ASL
                    STA        LSDF_Load_2+1
                    STZ        LSDF_Load_1+1
*--
LSDF_Load_1         LDX        #$0000               ; Load all Scene DataFile
LSDF_Load_2         CPX        #$AAAA               ; LSDF_DataFileNb * 2
                    BEQ        LSDF_End
                    LDA        LSDF_DataFileTab,X   ; DataFile ID 001-101
                    STA        LSDF_Load_File+1
*--
LSDF_Load_Check     JSR        CheckDataFileMem     ; Check if the DataFileID is already in memory
                    BEQ        LSDF_Load_Next       ; Already in Memory (Mem or Float)   
*--
LSDF_Load_Location  LDA        LSDF_Load_File+1     ; Search for a Bank where to load the file (Mem or Float)
                    LDX        LSDF_DataFileNb      ; Do not overwrite a Float Location we need 
                    LDY        #LSDF_DataFileTab
                    JSR        SearchLoadLocation   ;   => X = Bank, Y = @Location
                    STY        LSDF_Load_File_1+1
*--
LSDF_Load_File      LDA        #$AAAA               ; Load Data File in Memory
                    LDY        #$0001               ; Ask for Floppy if file not available 
                    JSR        LoadDataFile
                    BCS        LSDF_EndError
                    LDA        LSDF_Load_File+1     ; DataFile ID 001-101
LSDF_Load_File_1    STA        $AAAA
                    INC        LSDF_DataFileLoaded  ; Total number of loaded Datafile
*--
LSDF_Load_Next      INC        LSDF_Load_1+1        ; Next Datafile ID
                    INC        LSDF_Load_1+1
                    BRA        LSDF_Load_1                           
*--------
LSDF_End            LDA        LSDF_DataFileLoaded  ; Returns the number of loaded files
                    RTS
LSDF_EndError       LDA        #$FFFF               ; Fatal Error => Quit
                    RTS

*----------   LoadSceneDataFile Variables   ---------
LSDF_DataFileCurr   HEX        0000                 ; Current Data File
LSDF_DataFileNb     HEX        0000                 ; Number of Data File
LSDF_DataFileTab    HEX        0000,0000,0000,0000  ; 13 Floating Banks = Max number of Data files for 1 Scene
                    HEX        0000,0000,0000,0000
                    HEX        0000,0000,0000,0000
                    HEX        0000
LSDF_DataFileLoaded HEX        0000                 ; Number of Data File Loaded

LSDF_SceneID_Tab    HEX        0100,0200,0300,0400,0500,0600,0700,0800
                    HEX        0900,0A00,0B00,0C00,0D00,0E00,0F00,1000
                    HEX        1100,1200,1300,1400,1500,1600,1700,1800
                    HEX        1900,1A00,1B00,1C00,1D00,1E00,1F00,2000
                    HEX        2100,2200,2300,2500,2600,2700,2800,2900  ; Scene #24 is missing
                    HEX        2A00,2B00,2C00,2D00,2E00,2F00,3000,3100
                    HEX        3200,3300,3400,3C00     

LSDF_SceneID_Addr   DA         LSDF_01,LSDF_02,LSDF_03,LSDF_04,LSDF_05,LSDF_06,LSDF_07,LSDF_08
                    DA         LSDF_09,LSDF_0A,LSDF_0B,LSDF_0C,LSDF_0D,LSDF_0E,LSDF_0F,LSDF_10
                    DA         LSDF_11,LSDF_12,LSDF_13,LSDF_14,LSDF_15,LSDF_16,LSDF_17,LSDF_18
                    DA         LSDF_19,LSDF_1A,LSDF_1B,LSDF_1C,LSDF_1D,LSDF_1E,LSDF_1F,LSDF_20
                    DA         LSDF_21,LSDF_22,LSDF_23,LSDF_25,LSDF_26,LSDF_27,LSDF_28,LSDF_29
                    DA         LSDF_2A,LSDF_2B,LSDF_2C,LSDF_2D,LSDF_2E,LSDF_2F,LSDF_30,LSDF_31
                    DA         LSDF_32,LSDF_33,LSDF_34,LSDF_3C
                    
LSDF_01             HEX        0100,0000            ;  Required DataFileID for each Scene (including Death data)
LSDF_02             HEX        0100,0000
LSDF_03             HEX        0100,0200,0000
LSDF_04             HEX        0900,0000
LSDF_05             HEX        0100,0000
LSDF_06             HEX        0900,1000,1100,1200,1300,0000
LSDF_07             HEX        1200,1300,0000
LSDF_08             HEX        1300,1400,1500,1600,0000
LSDF_09             HEX        1600,1700,1500,0000
LSDF_0A             HEX        1700,1800,0000
LSDF_0B             HEX        0300,0400,0500,0600,0700,0800,0000
LSDF_0C             HEX        1800,1900,2000,0000
LSDF_0D             HEX        2000,2100,2200,0000
LSDF_0E             HEX        2300,2400,0000
LSDF_0F             HEX        2500,0000
LSDF_10             HEX        2500,2600,0000
LSDF_11             HEX        2600,2700,2800,2900,0000
LSDF_12             HEX        2900,3000,3200,3300,0000
LSDF_13             HEX        3000,3100,3200,3300,0000
LSDF_14             HEX        3300,0000
LSDF_15             HEX        3400,3500,3600,3700,0000
LSDF_16             HEX        3700,3800,3900,3600,0000
LSDF_17             HEX        3900,4000,4100,4200,4300,4400,3600,3700,0000
LSDF_18             HEX        4400,4500,4600,4700,4800,4900,5000,0000    
LSDF_19             HEX        5000,5100,0000
LSDF_1A             HEX        5100,5200,5300,5400,5500,0000
LSDF_1B             HEX        5500,5600,5400,0000
LSDF_1C             HEX        5700,5800,5400,5500,0000
LSDF_1D             HEX        5800,5900,6000,6100,6200,0000
LSDF_1E             HEX        6200,6300,6100,0000
LSDF_1F             HEX        6300,6400,6500,6100,6200,0000
LSDF_20             HEX        6500,6600,6700,6100,6200,0000
LSDF_21             HEX        6700,6800,6100,6200,0000
LSDF_22             HEX        6800,6900,0000
LSDF_23             HEX        6900,7000,0000
LSDF_25             HEX        7000,7100,7200,7300,7400,7500,0000
LSDF_26             HEX        7500,7600,7700,7800,0000
LSDF_27             HEX        7800,7900,8000,0000
LSDF_28             HEX        8100,8200,8300,8400,0000
LSDF_29             HEX        8300,8400,0000
LSDF_2A             HEX        8400,8500,0000
LSDF_2B             HEX        8500,0000
LSDF_2C             HEX        8600,8700,8200,0000
LSDF_2D             HEX        8700,8800,8300,8400,0000
LSDF_2E             HEX        8800,8900,8200,0000
LSDF_2F             HEX        8900,9000,8200,0000
LSDF_30             HEX        9000,9100,0000
LSDF_31             HEX        9100,9200,0000
LSDF_32             HEX        9200,9300,0000
LSDF_33             HEX        9300,9400,0000
LSDF_34             HEX        9400,9500,9600,0000
LSDF_3C             HEX        9600,0000
      

*********************************************************************************************
*  CheckDataFileMem() :  Check if DataFileID is already loaded in memory (IN:A=DataFileID). *
*********************************************************************************************
CheckDataFileMem    STA        CDFM_Memory_2+1      ; DataFileID
                    STA        CDFM_Float_2+1
                    LDA        NbAllocatedBank      ; NbAllocatedBank*2
                    ASL
                    STA        CDFM_Memory_3+1
                    LDA        REQUIRED_FLOAT_BANK  ; FLOATING_RAM_BANK*2
                    ASL
                    STA        CDFM_Float_3+1
*----                    
                    LDX        #$0000               ; Check if the DataFileID is already in Mem Banks
CDFM_Memory_1       LDA        BankData001ID,X
CDFM_Memory_2       CMP        #$AAAA               ; DataFile ID  001-101
                    BEQ        CDFM_Found           ; Already in Memory
                    INX
                    INX
CDFM_Memory_3       CPX        #$AAAA               ; NbAllocatedBank*2
                    BNE        CDFM_Memory_1
*----
                    LDX        #$0000               ; Check if the DataFileID is already in Float Banks
CDFM_Float_1        LDA        BankDataFloatID,X
CDFM_Float_2        CMP        #$AAAA               ; DataFile ID
                    BEQ        CDFM_Found           ; Already in Memory
                    INX
                    INX
CDFM_Float_3        CPX        #$AAAA               ; FLOATING_RAM_BANK*2
                    BNE        CDFM_Float_1                    
*--
CDFM_NotFound       LDA        #$0001               ; Not Found
                    RTS
CDFM_Found          LDA        #$0000               ; Found
                    RTS


************************************************************************************************************************************************
*  SearchLoadLocation() :  Search a place where to Load the File (IN:A=DataFileID,IN:X=NbDataFile,IN:Y=TabDataFile,OUT:X=Bank,OUT:Y=Location). * 
************************************************************************************************************************************************
SearchLoadLocation  STA        SLL_MemFloat+1       ; DataFile ID (Decimal)
                    STA        SLL_Memory+1
                    STX        SLL_Float_2+1        ; Number of ID in DataFileID for this Scene
                    STY        SLL_Float_3+1        ; Table of DataFileID for this Scene
*----
SLL_MemFloat        LDA        #$AAAA               ; Is it in the Mem Banks or in Float Banks ?
                    JSR        HexDec2Hex           ; DataFile ID (Hexadecimal)
                    CMP        NbAllocatedBank
                    BEQ        SLL_Memory           ; Direct Location in Memory
                    BMI        SLL_Memory
*----
SLL_Float           STZ        SLL_Float_1+1        ; We have to load the DataFile into a Float Bank
SLL_Float_1         LDX        #$0000               ; Loop over all avaiable Float Banks
                    LDA        BankDataFloatID,X    ; DataFileID stored in this Bank
                    STX        SLL_Float_6+1
SLL_Float_2         LDX        #$AAAA               ; Number ID required for this Scene
SLL_Float_3         LDY        #$BBBB               ; @ ID Table containing all required DataFile ID for the Scene
                    JSR        SearchIDIntoTable    ; Do we need this DataFileID later ?
                    BNE        SLL_Float_6          ; We don't need this ID for the Scene => Take this Bank / Slot
                    INC        SLL_Float_1+1        ; Next Float Bank
                    INC        SLL_Float_1+1
                    BRA        SLL_Float_1                    
*--
SLL_Float_6         LDY        #$AAAA               ; Keep this Bank / Location for this DataFile
                    LDA        BankDataFloat,y
                    TAX                             ; X = Bank Number
                    TYA
                    STA        SLL_Float_7+1
                    LDA        #BankDataFloatID
                    CLC
SLL_Float_7         ADC        #$BBBB 
                    TAY                             ; Y = Location @    
                    BRA        SLL_End
*----                    
SLL_Memory          LDA        #$AAAA               ; Direct Location in Memory Bank
                    JSR        HexDec2Hex           ; DataFile ID (Hexa)
                    DEC
                    ASL
                    STA        SLL_Memory_1+1
                    TAX
                    LDA        BankData001,X        
                    TAX                             ; X = Bank Number 
                    LDA        #BankData001ID       ; @Location for DataFileID
                    CLC
SLL_Memory_1        ADC        #$AAAA
                    TAY                             ; Y = Location @
*----                    
SLL_End             RTS


***********************************************************************************************************************************
*  SearchIDIntoTable() :  Search for an ID into an ID Table (IN:A=ID,IN:X=NbDataFile,IN:Y=TabDataFile,OUT:X=Bank,OUT:A=0IfFound). *
***********************************************************************************************************************************
SearchIDIntoTable   STA        SIDIT_Search_2+1     ; ID
                    STY        SIDIT_Search_1+1     ; @ ID Table
                    TXA                             ; Number ID in Table
                    ASL
                    STA        SIDIT_Search_3+1
*--
                    LDX        #$0000               ; Search ID into Table 
SIDIT_Search_1      LDA        $AAAA,X
SIDIT_Search_2      CMP        #$BBBB
                    BEQ        SIDIT_Found
                    INX
                    INX
SIDIT_Search_3      CPX        #$AAAA
                    BNE        SIDIT_Search_1
*--
SIDIT_NotFound      LDA        #$0001               ; Not Found
                    RTS
SIDIT_Found         LDA        #$0000               ; Found
                    RTS


*******************************************************************************************
*  BuildFileDataPath() :  Build the File Disk*/Data***.bin path (IN:FilePath,IN:X=File#). *
*******************************************************************************************
BuildFileDataPath   CLC                             ; A = File Data Path
                    ADC        #$0008               ; 1/DiskX
                    STA        BFDP_Disk_3+1
                    ADC        #$0006 
                    STA        BFDP_DataNum_3+1     ; DataXXX.bin
                    INC
                    STA        BFDP_DataNum_4+1        
                    STX        BFDP_FileNum         ; X = File Number 001-101
*--
                    LDX        #$0000               ; Identify Disk Number
BFDP_Disk_1         LDA        Scene_01+1,X         ; DataFile ID
                    XBA                             ; is stored in Big Endian
                    CMP        BFDP_FileNum
                    BEQ        BFDP_Disk_2
                    TXA
                    CLC
                    ADC        #$000A               ; Next Line
                    TAX
                    BRA        BFDP_Disk_1
BFDP_Disk_2         LDA        Scene_01,X           ; Disk #
                    AND        #$00FF               ; 01 -> 07
                    CLC
                    ADC        #$2F30               ; '/1' -> '/7'
BFDP_Disk_3         STA        $AAAA                ; FilePath + 8
*--
                    LDX        #$FFFE               ; Build File Data Number in Ascii
BFDP_DataNum_1      INX
                    INX
                    LDA        BFDP_FileNumHEX,X
                    CMP        BFDP_FileNum         ; File Data Number 001-101
                    BNE        BFDP_DataNum_1
                    TXA
                    STA        BFDP_DataNum_2+1
                    LSR
                    CLC
BFDP_DataNum_2      ADC        #$AAAA               ; *3
                    TAX
                    LDA        BFDP_FileNumAscii,X
BFDP_DataNum_3      STA        $AAAA                ; File Path + 14
                    LDA        BFDP_FileNumAscii+1,X
BFDP_DataNum_4      STA        $AAAA                ; File Path + 15
*--
                    RTS
                    
*----------   BuildFileDataPath Variables   ---------
BFDP_FileNum        HEX        0000
BFDP_FileNumHEX     HEX        0000,0100,0200,0300,0400,0500,0600,0700,0800,0900,1000,1100,1200,1300,1400,1500,1600,1700,1800,1900
                    HEX        2000,2100,2200,2300,2400,2500,2600,2700,2800,2900,3000,3100,3200,3300,3400,3500,3600,3700,3800,3900
                    HEX        4000,4100,4200,4300,4400,4500,4600,4700,4800,4900,5000,5100,5200,5300,5400,5500,5600,5700,5800,5900
                    HEX        6000,6100,6200,6300,6400,6500,6600,6700,6800,6900,7000,7100,7200,7300,7400,7500,7600,7700,7800,7900
                    HEX        8000,8100,8200,8300,8400,8500,8600,8700,8800,8900,9000,9100,9200,9300,9400,9500,9600,9700,9800,9900
                    HEX        0001,0101       
BFDP_FileNumAscii   ASC        '000001002003004005006007008009010011012013014015016017018019'
                    ASC        '020021022023024025026027028029030031032033034035036037038039'
                    ASC        '040041042043044045046047048049050051052053054055056057058059'
                    ASC        '060061062063064065066067068069070071072073074075076077078079'
                    ASC        '080081082083084085086087088089090091092093094095096097098099'
                    ASC        '100101'


*************************************************************************************************************
*  GetSceneOffset() :  Get Scene Offset/Length (IN:A=Scene,IN:X=Type,IN:Y=Frame,OUT:X=Offset,OUT:Y=Length). *
*************************************************************************************************************
GetSceneOffset      STA        GSO_Scene            ; Init
                    STX        GSO_Type 
                    STY        GSO_Frame
*--
                    LDX        #$000                ; Get Scene Base Address
GSO_Scene_1         LDA        TAB_SceneID,X
                    CMP        GSO_Scene
                    BEQ        GSO_Scene_2
                    INX
                    INX
                    BRA        GSO_Scene_1
GSO_Scene_2         LDA        TAB_SceneAddr,X      ; Scene Base Address
                    CLC
                    ADC        #$0003               ; Scene Base Address+3 = Offset
                    STA        GSO_Offset+1
                    ADC        #$0002               ; Scene Base Address+5 = Length
                    STA        GSO_Length+1
                    ADC        #$0003               ; Scene Base Address+8 = Type/Frame
                    STA        GSO_TypeFrame_1+1
*--
                    SEP        #$20                 ; Identify Type / Frame
                    LDA        GSO_Frame
                    XBA
                    LDA        GSO_Type
                    REP        #$30                 ; A = FrameType
                    STA        GSO_TypeFrame_2+1
*--
                    LDX        #$0000
GSO_TypeFrame_1     LDA        $AAAA,X              ; Scene Type/Frame
GSO_TypeFrame_2     CMP        #$BBBB               ; FrameType
                    BEQ        GSO_Length
                    TXA
                    CLC
                    ADC        #$000A               ; Line Length
                    TAX
                    BRA        GSO_TypeFrame_1
*--
GSO_Length          LDA        $AAAA,X              ; Scene Length
                    XBA                             ; Stored as Big Endian in the table
                    TAY
GSO_Offset          LDA        $AAAA,X              ; Scene Offset
                    XBA                             ; Stored as Big Endian in the table
                    TAX                
*--
                    RTS     

*----------   GetSceneOffset Variables   ---------
GSO_Scene           HEX        0000                 ; 0001 - 003C
GSO_Type            HEX        0000                 ; 0000=GameBck, 0001=GameFrm, 0002=GameSnd, 0080=DeathBck, 0081=DeathFrm, 0082=DeathSnd
GSO_Frame           HEX        0000                 ; 0001 - XXXX


******************************************************************************************************************
*  CopySoundDataToRAM() :  Copy/Uncompress Sound Data to Sound RAM (IN:A=Sound@Bank,IN:X=Sound@,IN:Y=SoundSize). *
******************************************************************************************************************
CopySoundDataToRAM  STA        CSDTR_1+2            ; Sound Bank Address A=XX00
                    STA        CSDTR_2+2
                    STX        CSDTR_1+1            ; Sound Address
                    STX        CSDTR_2+1
                    TYA                             ; Sound Length
                    LSR                             ; Force Even Length
                    ASL
                    CMP        #$7FFA
                    BLT        CSDTR_Init           ; Make sure the Sound Length < 64 KB
                    LDA        #$7FFA
CSDTR_Init          STA        CSDTR_4+1            ; Sound Length
*--
                    LDA        SoundToolError       ; No Sound ?
                    BEQ        CSDTR_CC_SS
                    JMP        CSDTR_END
*--
CSDTR_CC_SS         JSR        StopSound            ; Stop Current Sound
*--
                    SEI                             ; Disable Interrupts
                    PHD                             ; Save Direct Page Address
                    LDA        #$C000
                    TCD                             ; Define $C000 as the new Direct Page Address
*--
                    SEP        #$20
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F
                    ORA        #$60                 ; Access Sound Ram + Auto Increment Enabled
                    STAL       SOUNDCTL
                    LDA        #$00
                    STAL       SOUNDADRL            ; Sound Ram Address = $0000
                    STAL       SOUNDADRH
                    XBA                             ; Make sure B is #$00 for the TAY               
*--
                    LDA        #$00                 ; Make sure a Sound starting at $8000 will stop immediately
                    STAL       SOUNDADRL            ; Sound Ram Address = $8000
                    LDA        #$80
                    STAL       SOUNDADRH
                    LDA        #$00
                    STA        $3D                  ; SOUNDDATA = $C03D
                    STA        $3D                  
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
*--
                    LDA        #$00                 ; Copy Sound Data into Sound RAM at $0000
                    STAL       SOUNDADRL            ; Sound Ram Address = $0000
                    STAL       SOUNDADRH
                    XBA                             ; Make sure B is #$00 for the TAY               
                    LDX        #$0000
CSDTR_1             LDAL       $BBAAAA,X            ; Copy/Uncompress Sound Data B795->B975->B0 90 70 50 
                    TAY
                    LDA        CSDTR_UPPER_NIBBLE,Y
                    STA        $3D                  ; SOUNDDATA = $C03D
                    LDA        CSDTR_LOWER_NIBBLE,Y
                    STA        CSDTR_3+1
                    INX
CSDTR_2             LDAL       $BBAAAA,X  
                    TAY
                    LDA        CSDTR_UPPER_NIBBLE,Y
                    STA        $3D                  ; SOUNDDATA = $C03D
CSDTR_3             LDA        #$AA
                    STA        $3D                  ; SOUNDDATA = $C03D
                    LDA        CSDTR_LOWER_NIBBLE,Y
                    STA        $3D                  ; SOUNDDATA = $C03D
                    INX
CSDTR_4             CPX        #$AAAA               ; Sound Size
                    BNE        CSDTR_1
*--
                    LDA        #$00                 ; Add few #$00 to stop the Sound
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
*--
                    REP        #$30
                    PLD                             ; Restore Direct Page Address
                    CLI                             ; Enable Interrupts
CSDTR_END           RTS                    
                    
*----------   CopySoundDataToRAM Variables   ---------
CSDTR_UPPER_NIBBLE  HEX        01,01,01,01,01,01,01,01,01,01,01,01,01,01,01,01
                    HEX        10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10
                    HEX        20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20
                    HEX        30,30,30,30,30,30,30,30,30,30,30,30,30,30,30,30
                    HEX        40,40,40,40,40,40,40,40,40,40,40,40,40,40,40,40
                    HEX        50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50
                    HEX        60,60,60,60,60,60,60,60,60,60,60,60,60,60,60,60
                    HEX        70,70,70,70,70,70,70,70,70,70,70,70,70,70,70,70
                    HEX        80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80
                    HEX        90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90
                    HEX        A0,A0,A0,A0,A0,A0,A0,A0,A0,A0,A0,A0,A0,A0,A0,A0
                    HEX        B0,B0,B0,B0,B0,B0,B0,B0,B0,B0,B0,B0,B0,B0,B0,B0
                    HEX        C0,C0,C0,C0,C0,C0,C0,C0,C0,C0,C0,C0,C0,C0,C0,C0
                    HEX        D0,D0,D0,D0,D0,D0,D0,D0,D0,D0,D0,D0,D0,D0,D0,D0
                    HEX        E0,E0,E0,E0,E0,E0,E0,E0,E0,E0,E0,E0,E0,E0,E0,E0
                    HEX        F0,F0,F0,F0,F0,F0,F0,F0,F0,F0,F0,F0,F0,F0,F0,F0

CSDTR_LOWER_NIBBLE  HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    HEX        01,10,20,30,40,50,60,70,80,90,A0,B0,C0,D0,E0,F0
                    
                    
*************************************************************************************************************
*  PlaySoundData() :  Start ENSONIQ Oscillators 0 -> 3 to play Sound Sample located at $0000 (IN:A=SceneID). *
*************************************************************************************************************
PlaySoundData       STA        PSD_Init_2+1         ; Search for Frequency
                    LDX        #$0000
PSD_Init_1          LDA        PSD_TabSceneID,X
PSD_Init_2          CMP        #$AAAA
                    BEQ        PSD_Init_3
                    INX
                    INX        
                    BRA        PSD_Init_1
PSD_Init_3          LDA        PSD_TabFrequency,X
                    SEP        #$20
                    STA        PSD_Frequency_Lo_0+1
                    STA        PSD_Frequency_Lo_1+1
                    STA        PSD_Frequency_Lo_2+1
                    STA        PSD_Frequency_Lo_3+1
                    XBA
                    STA        PSD_Frequency_Hi_0+1
                    STA        PSD_Frequency_Hi_1+1
                    STA        PSD_Frequency_Hi_2+1
                    STA        PSD_Frequency_Hi_3+1
*--
                    LDA        SoundToolError       ; No Sound ?
                    BEQ        PSD_Init_SS
                    JMP        PSD_End
*--
PSD_Init_SS         REP        #$30
                    JSR        StopSound            ; Stop Current sound
                    SEP        #$20
*--
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F                 ; Access Sound Register + Auto Increment Disabled           
                    STAL       SOUNDCTL
*--
PSD_Enable          LDA        #$E1                 ; Oscillators Enable Register
                    STAL       SOUNDADRL
                    LDA        #$3E                 ; # Oscillators x 2 => We want all the 32 oscillators running
                    STAL       SOUNDDATA
*----
PSD_Frequency       LDA        #$00                 ; Frequency Low Register for Oscillator #00
                    STAL       SOUNDADRL
PSD_Frequency_Lo_0  LDA        #$D6                 ; Frequency Low value (Hz*32/1645), 11025 hz => 214 = D6
                    STAL       SOUNDDATA
                    LDA        #$20                 ; Frequency High Register for Oscillator #00
                    STAL       SOUNDADRL
PSD_Frequency_Hi_0  LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA
*--
                    LDA        #$01                 ; Frequency Low Register for Oscillator #01
                    STAL       SOUNDADRL
PSD_Frequency_Lo_1  LDA        #$D6                 ; Frequency Low value (Hz*32/1645), 11025 hz => 214
                    STAL       SOUNDDATA
                    LDA        #$21                 ; Frequency High Register for Oscillator #01
                    STAL       SOUNDADRL
PSD_Frequency_Hi_1  LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA                    
*--
                    LDA        #$02                 ; Frequency Low Register for Oscillator #02
                    STAL       SOUNDADRL
PSD_Frequency_Lo_2  LDA        #$D6                 ; Frequency Low value (Hz*32/1645), 11025 hz => 214
                    STAL       SOUNDDATA
                    LDA        #$22                 ; Frequency High Register for Oscillator #02
                    STAL       SOUNDADRL
PSD_Frequency_Hi_2  LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA                    
*--
                    LDA        #$03                 ; Frequency Low Register for Oscillator #03
                    STAL       SOUNDADRL
PSD_Frequency_Lo_3  LDA        #$D6                 ; Frequency Low value (Hz*32/1645), 11025 hz => 214
                    STAL       SOUNDDATA
                    LDA        #$23                 ; Frequency High Register for Oscillator #03
                    STAL       SOUNDADRL
PSD_Frequency_Hi_3  LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA                    
*----                    
PSD_Volume          LDA        #$40                 ; Volume Register for Oscillator #00
                    STAL       SOUNDADRL
                    JSR        GetMuteStatus        ; Volume : 00 (Min) / FF (Max)
                    STAL       SOUNDDATA
*--
                    LDA        #$41                 ; Volume Register for Oscillator #01
                    STAL       SOUNDADRL
                    JSR        GetMuteStatus        ; Volume : 00 (Min) / FF (Max)
                    STAL       SOUNDDATA
*--
                    LDA        #$42                 ; Volume Register for Oscillator #02
                    STAL       SOUNDADRL
                    JSR        GetMuteStatus        ; Volume : 00 (Min) / FF (Max)
                    STAL       SOUNDDATA
*--
                    LDA        #$43                 ; Volume Register for Oscillator #03
                    STAL       SOUNDADRL
                    JSR        GetMuteStatus        ; Volume : 00 (Min) / FF (Max)
                    STAL       SOUNDDATA
*----
PSD_Address         LDA        #$80                 ; Sound Address Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Sound Address = $0000 => $00
                    STAL       SOUNDDATA         
*--
                    LDA        #$81                 ; Sound Address Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDA        #$80                 ; Sound Address = $8000 => $80
                    STAL       SOUNDDATA
*--
                    LDA        #$82                 ; Sound Address Register for Oscillator #02
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Sound Address = $0000 => $00
                    STAL       SOUNDDATA
*--
                    LDA        #$83                 ; Sound Address Register for Oscillator #03
                    STAL       SOUNDADRL
                    LDA        #$80                 ; Sound Address = $8000 => $80
                    STAL       SOUNDDATA
*----
PSD_Size            LDA        #$C0                 ; Size Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDA        #$3F                 ; Sound Size = 32 KB
                    STAL       SOUNDDATA
*--
                    LDA        #$C1                 ; Size Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDA        #$3F                 ; Sound Size = 32 KB
                    STAL       SOUNDDATA                    
*--
                    LDA        #$C2                 ; Size Register for Oscillator #02
                    STAL       SOUNDADRL
                    LDA        #$3F                 ; Sound Size = 32 KB
                    STAL       SOUNDDATA                    
*--
                    LDA        #$C3                 ; Size Register for Oscillator #03
                    STAL       SOUNDADRL
                    LDA        #$3F                 ; Sound Size = 32 KB
                    STAL       SOUNDDATA                    
*----
PSD_Control         LDA        #$A0                 ; Control Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDA        #$06                 ; Canal #0=0000 / No Interrupt=0 / Swap=11 / Start Sound=0
                    STAL       SOUNDDATA
*--
                    LDA        #$A1                 ; Control Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Canal #1=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
*--
                    LDA        #$A2                 ; Control Register for Oscillator #02
                    STAL       SOUNDADRL
                    LDA        #$16                 ; Canal #1=0001 / No Interrupt=0 / Swap=11 / Start Sound=0
                    STAL       SOUNDDATA
*--
                    LDA        #$A3                 ; Control Register for Oscillator #03
                    STAL       SOUNDADRL
                    LDA        #$13                 ; Canal #1=0001 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
*----
PSD_End             REP        #$30
                    RTS                 

*----------   PlaySoundData Variables   ---------                     
PSD_TabSceneID      HEX        0300,0600,0680,0700,0780,0800,0880,0900,0A00,0B00
                    HEX        0B80,0C00,0C80,0D00,0D80,0E00,0E80,0F00,1000,1100
                    HEX        1200,1300,1380,1400,1500,1580,1600,1700,1800,1880
                    HEX        1900,1A00,1A80,1B00,1C00,1D00,1D80,1E00,1F00,2000
                    HEX        2100,2200,2300,2500,2580,2600,2680,2700,2780,2800
                    HEX        2880,2900,2980,2A00,2B00,2C00,2D00,2E00,2F00,3000
                    HEX        3100,3200,3300,3400
                                                                                           
PSD_TabFrequency    HEX        7400,9200,9B00,9200,9B00,9B00,9B00,A200,9B00,9B00   ; FreqOsc=FreqHz*32/1645
                    HEX        9B00,7400,9B00,A200,9B00,A200,9B00,7400,A200,A200
                    HEX        A200,A200,9B00,A200,A200,9B00,9B00,9B00,A200,9B00
                    HEX        A200,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00
                    HEX        9B00,9B00,A200,9200,9B00,A200,9B00,9B00,9B00,9B00
                    HEX        9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,A200
                    HEX        A200,A200,9B00,9B00
                               
                                          
***********************************************************************************
*  WaitEndOfSound() :  Wait until the Sound stop playing from Oscillators 0 -> 3. *
***********************************************************************************
WaitEndOfSound      LDA        SoundToolError       ; No Sound ?
                    BEQ        WEOS_Init
                    JMP        WEOS_End
*--
WEOS_Init           SEP        #$20
*--
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F                 ; Access Sound Register + Auto Increment Disabled           
                    STAL       SOUNDCTL                    
*--                                     
                    LDA        #$A0                 ; Control Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDX        #$0000
WEOS_OSC0           LDAL       SOUNDDATA            ; Oscillator #0 information
                    LSR
                    BCC        WEOS_OSC0            ; Bit #0 = 0 => Still Active
                    INX
                    CPX        #$8000
                    BNE        WEOS_OSC0            ; Wait for #$8000 non active sequences
*--
                    LDA        #$A1                 ; Control Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDX        #$0000
WEOS_OSC_1          LDAL       SOUNDDATA            ; Oscillator #1 information
                    LSR
                    BCC        WEOS_OSC_1           ; Bit #0 = 0 => Still Active
                    INX
                    CPX        #$8000
                    BNE        WEOS_OSC_1           ; Wait for #$8000 non active sequences
*--
                    LDA        #$A2                 ; Control Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDX        #$0000
WEOS_OSC_2          LDAL       SOUNDDATA            ; Oscillator #2 information
                    LSR
                    BCC        WEOS_OSC_2           ; Bit #0 = 0 => Still Active
                    INX
                    CPX        #$8000
                    BNE        WEOS_OSC_2           ; Wait for #$8000 non active sequences
*--
                    LDA        #$A3                 ; Control Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDX        #$0000
WEOS_OSC_3          LDAL       SOUNDDATA            ; Oscillator #3 information
                    LSR
                    BCC        WEOS_OSC_3           ; Bit #0 = 0 => Still Active
                    INX
                    CPX        #$8000
                    BNE        WEOS_OSC_3           ; Wait for #$8000 non active sequences
*--
                    REP        #$30
WEOS_End            RTS  


********************************************************************************************************
*  GetFrameBank() :  Identify the Memory Bank hosting the SceneTypeFrame (IN:Y=@SceneLine,OUT:A=Bank). *
********************************************************************************************************
GetFrameBank        LDA:       $0001,Y              ; Disk(1)Data(2)Offset(2)Length(2)Scene(1)Type(1)Frame(1)
                    XBA                             ; Data ID is stored in big endian in the table
                    STA        GetFrameBank_2+1
                    STA        GetFrameBank_4+1
                    PHX
                    PHY
*--
                    LDX        #$0000               ; Search In Ram Banks
                    LDY        #$0000
GetFrameBank_1      LDA        BankData001ID,X
GetFrameBank_2      CMP        #$AAAA               ; DataFile ID
                    BEQ        GetFrameBank_Ram
                    INX
                    INX
                    INY
                    CPY        NbAllocatedBank
                    BNE        GetFrameBank_1
*--
                    LDX        #$0000               ; Search In Float Banks
                    LDY        #$0000
GetFrameBank_3      LDA        BankDataFloatID,X
GetFrameBank_4      CMP        #$AAAA               ; DataFile ID
                    BEQ        GetFrameBank_Float
                    INX
                    INX
                    INY
                    CPY        REQUIRED_FLOAT_BANK
                    BNE        GetFrameBank_3
*--
                    PLY                             ; Not Found : Never Here !
                    PLX
                    BRK        $BD
*--
GetFrameBank_Ram    LDA        TableAllocatedBank,X ; Found in Ram Bank
                    BRA        GetFrameBank_End
*--
GetFrameBank_Float  LDA        BankDataFloat,X      ; Found in Float Bank
*--
GetFrameBank_End    PLY
                    PLX
                    RTS


********************************************************************************************
*  GetPictureBank() :  Get Memory Bank for a Background Picture (IN:A=SceneID,OUT:A=Bank). *
********************************************************************************************
GetPictureBank      STA        GPB_Addr_1+1         ; Scene ID
                    LDA        NbAllocatedBank
                    ASL
                    STA        GPB_Bank_Ram_2+1
                    LDA        REQUIRED_FLOAT_BANK
                    ASL
                    STA        GPB_Bank_Float_2+1
*--
                    LDX        #$0000
GPB_Addr_0          LDA        TAB_SceneID,X
GPB_Addr_1          CMP        #$AAAA
                    BEQ        GPB_DataFileID
                    INX
                    INX
                    BRA        GPB_Addr_0
*--
GPB_DataFileID      LDA        TAB_SceneAddr,X      ; Scene Line Address
                    TAX
                    LDA:       $0001,X              ; DataFile ID
                    XBA                             ; Stored in the table using Big Endian        
                    STA        GPB_Bank_Ram_1+1
                    STA        GPB_Bank_Float_1+1
*--
                    LDX        #$0000               ; Ram Bank
GPB_Bank_Ram_0      LDA        BankData001ID,X
GPB_Bank_Ram_1      CMP        #$BBBB
                    BEQ        GPB_Bank_Ram
                    INX
                    INX
GPB_Bank_Ram_2      CPX        #$CCCC
                    BNE        GPB_Bank_Ram_0
*--
                    LDX        #$0000               ; Ram Float
GPB_Bank_Float_0    LDA        BankDataFloatID,X
GPB_Bank_Float_1    CMP        #$BBBB
                    BEQ        GPB_Bank_Float
                    INX
                    INX
GPB_Bank_Float_2    CPX        #$CCCC
                    BNE        GPB_Bank_Float_0                    
*--
GPB_Bank_Ram        LDA        BankData001,X        ; A = XX00
                    BRA        GPB_End
GPB_Bank_Float      LDA        BankDataFloat,X            
*--
GPB_End             RTS


**********************************************************
*  CopyPixelSCB()  :  Copy Pixel + SCB data to $01/2000. *
**********************************************************
CopyPixelSCB        LDA        CurrentBank          ; 00AA
                    ORA        #$0100
                    XBA
                    STA        CPSCB_Bank+1
*--                    
                    LDA        #$7DFF               ; Copy 7D00+100 bytes
                    LDX        #BUFFERPIXEL         ; Source
                    LDY        #$2000               ; Destination
CPSCB_Bank          MVN        $BB,$01
                    PHK
                    PLB
                    RTS


*************************************************************************************
*  PlayScene() :  Run animation for Scene #N (IN:A=SceneNumber,IN:X=IsAttractMode). *
*************************************************************************************
PlayScene           STA        PS_SceneID           ; 0006 -> 003E
                    STX        PS_IsAttractMode
                    STZ        PS_SceneOffset  
PS_Init             LDX        #PS_SceneNbScreen    ; Get Scene Information
                    JSR        GetSceneInfo
                    STZ        PS_SceneFrame        ; Current Frame Number 1-N
                    STZ        PS_FrameHasHint      ; Display Hint on this Frame
                    STZ        PS_UserAction        ; No User Action yet
                    JSR        ClearKDBStrobe
                    LDA        #$0000
                    JSR        SetSaveLevel         ; No save at the end of the Scene
                    
*----  Game Action List : Get Action List + Build Timelapse                    
                    LDA        PS_SceneID           ; Scene ID
                    LDX        #PS_SceneNbAction
                    JSR        GetSceneAction       ; Get Scene Action List
*--
                    LDX        #$01A2               ; 3*70*2 -2
PS_Action_Reset     STZ        PS_ActionTimeLapse,X
                    DEX
                    DEX
                    BPL        PS_Action_Reset
                    LDX        #$0002               ; Reset Forbidden Moves                    
PS_Forbidden_Reset  STZ        PS_ForbiddenMoves,X
                    DEX
                    DEX
                    BPL        PS_Forbidden_Reset
*--
                    LDY        #$0000
PS_Action_Frame     LDA        PS_SceneAction+0,Y   ; Frame #
                    BEQ        PS_Action_Frame_1
                    CMP        #$00FF               ; Forbidden Moves
                    BNE        PS_Action_Frame_0
*--
                    LDA        PS_ForbiddenMoves    ; No more than 2 Forbidden Moves per Scene
                    BNE        PS_Forbidden_1
                    LDA        PS_SceneAction+2,Y   ; Action
                    STA        PS_ForbiddenMoves+0
                    LDA        PS_SceneAction+4,Y   ; Death ID
                    STA        PS_ForbiddenMoves+2 
                    BRA        PS_Action_Frame_1  
PS_Forbidden_1      LDA        PS_SceneAction+2,Y   ; Action
                    STA        PS_ForbiddenMoves+4
                    LDA        PS_SceneAction+4,Y   ; Death ID
                    STA        PS_ForbiddenMoves+6 
                    BRA        PS_Action_Frame_1
*--                    
PS_Action_Frame_0   DEC
                    ASL
                    TAX
                    LDA        PS_SceneAction+2,Y   ; Action
                    STA        PS_ActionTimeLapse,X
                    LDA        PS_SceneAction+4,Y   ; Death ID
                    STA        PS_DeathTimeLapse,X
*-- 
PS_Action_Frame_1   TYA                             ; Next PS_SceneAction line
                    CLC
                    ADC        #$0006               
                    TAY
                    CPY        #$001E
                    BNE        PS_Action_Frame
*--                    
                    LDX        #$0000               ; Insert Hint into TimeLapse
PS_Action_Hint      LDA        PS_ActionTimeLapse,X
                    BEQ        PS_Action_Hint_5
                    STA        PS_Action_Hint_2+1
                    STX        PS_Action_Hint_4+1
                    LDY        #$0000
                    CPX        #$000C               ; 6*2 Display Hint in the 6 previous frames
                    BLT        PS_Action_Hint_1
                    TXA
                    SEC
                    SBC        #$000C               ; 6*2
                    TAY                             ; Y = First Scene Index
PS_Action_Hint_1    LDA        PS_HintTimeLapse,Y
                    BNE        PS_Action_Hint_3     ; Do not overwrite previous Hint
PS_Action_Hint_2    LDA        #$BBBB               ; Action
                    STA        PS_HintTimeLapse,Y
PS_Action_Hint_3    INY
                    INY
PS_Action_Hint_4    CPY        #$AAAA
                    BNE        PS_Action_Hint_1
*--                    
PS_Action_Hint_5    INX
                    INX
                    CPX        #$008C               ; 70*2 Frame Max
                    BNE        PS_Action_Hint
                       
*----  Game Background : Decompress Game Screen Pixels  ----
PS_GameBackground   JSR        FadeOut              ; FadeOut to Black
*--
                    LDA        #$0000
                    JSR        SetSaveLevel         ; No save at the end of the Scene
*--
                    LDX        PS_SceneOffset
                    LDA        PS_SceneScreen+0,X   ; Game Screen Offset
                    TAY
*--
                    JSR        GetFrameBank         ; Get Frame Data Bank
                    STA        PS_FrameBank
                    LDA:       $0003,Y              ; X = Offset
                    XBA
                    TAX
                    LDA:       $0005,Y              ; Y = Length
                    XBA
                    TAY
                    LDA        PS_FrameBank         ; A = AA00 Bank Hosting Frame Data
                    ORA        CurrentBank          ; 00BB               
                    JSR        UnpackLZ4            ; A = BankSrc,BankDst / X = 1st Packed Byte offset / Y = Pack Data Size
                    JSR        CopyPixelSCB         ; Copy Pixels + SCB to $01/2000

*----  Game Frame #0 : Add Frame #0 Pixel Data  ----
PS_GameFrame0       LDX        PS_SceneOffset       ; Display Frame #0 Bitmap
                    LDA        PS_SceneScreen+4,X   ; Frame #0 Offset
                    TAY
PS_UnpackFrame0     JSR        GetFrameBank         ; Get Frame Data Bank
                    STA        PS_FrameBank
                    LDA:       $0003,Y              ; Unpack Picture Frame
                    XBA
                    TAX                             ; X = Offset
                    LDA:       $0005,Y              ; Y = Length
                    XBA
                    TAY
                    LDA        PS_FrameBank         ; A = AA00 Bank Hosting Scene Data
                    JSR        UnpackFrame

*----  Attract Message (in needed)  ----
PS_AttractMessage0  LDA        PS_IsAttractMode     ; Add Attract Message at the Top of the Screen
                    BEQ        PS_FadeIn
*--
PS_AM0_RemapColor_1 LDA        PS_SceneID           ; Remap Colors for Scene #0B
                    CMP        #$000B
                    BNE        PS_AM0_Display
                    LDX        #PressFireButton     ; @Data
                    LDY        #$03A8               ; Data Length
                    LDA        #$2255               ; Color 2 -> 5
                    JSL        RemapBitmapColor
                    LDX        #PressFireButton     ; @Data
                    LDY        #$03A8               ; Data Length
                    LDA        #$1144               ; Color 1 -> 4
                    JSL        RemapBitmapColor
*--
PS_AM0_Display      LDX        #PressFireButton
                    LDY        #$2160
                    LDA        #$3409
                    CLC                             ; 01/2000
                    JSL        CopyTransBitmap
*--                    
PS_AM0_RemapColor_2 LDA        PS_SceneID           ; Remap Back Colors for Scene #0B
                    CMP        #$000B
                    BNE        PS_FadeIn
                    LDX        #PressFireButton     ; @Data
                    LDY        #$03A8               ; Data Length
                    LDA        #$4411               ; Color 4 -> 1
                    JSL        RemapBitmapColor 
                    LDX        #PressFireButton     ; @Data
                    LDY        #$03A8               ; Data Length
                    LDA        #$5522               ; Color 5 -> 2
                    JSL        RemapBitmapColor 
                    
*----  Fade In  ----
PS_FadeIn           LDA        #BUFFER32KB          ; FadeIn to Game Screen
                    LDY        #$FFFF               ; Don't Copy Pixel, Only Palette
                    JSR        FadeIN

*----  Play Sound (if there)  ----                    
PS_GameSound        LDY        #$0000
                    LDX        #$0000
PS_SearchSound      LDA        PS_SceneScreen+6,X   ; Game Sound DataLine Offset
                    BNE        PS_PlayGameSound
                    TXA                             ; Next Screen offset
                    CLC
                    ADC        #$0008
                    TAX
                    INY
                    CPY        PS_SceneNbScreen
                    BNE        PS_SearchSound
                    BRA        PS_GameScene1N       ; No Sound for this scene                    
*--
PS_PlayGameSound    TAY
                    JSR        GetFrameBank         ; Get Frame Data Bank
                    STA        PS_FrameBank                    
                    LDA:       $0003,Y              ; X = Sound Address
                    XBA
                    TAX
                    LDA:       $0005,Y              ; Y = Sound Length
                    XBA
                    TAY
                    LDA        PS_FrameBank         ; A = AA00 Bank Hosting Frame Data
                    JSR        CopySoundDataToRAM   ; Copy Sound Data to Ram Sound
                    LDA        PS_SceneID           ; Scene ID
                    JSR        PlaySoundData        ; Start playing Sound

*----  Play Game Frames 1-N  ----
PS_GameScene1N      LDX        PS_SceneOffset       ; Init Display Frame #1-N
                    LDA        PS_SceneScreen+2,X   ; Number of Game Frame
                    STA        PS_LoopFrame+1
                    LDA        PS_SceneScreen+4,X   ; Frame #N Offset
                    STA        PS_UnpackFrame+1
*--
PS_InitTimer        JSR        HBResetCounter       ; Reset VBL counter
                    INC        PS_SceneFrame        ; PS_SceneFrame = 1
                    JMP        PS_GetUserAction     ; The first frame is already on the screen, so wait
*----
PS_LoopFrame        LDA        #$AAAA               ; Number of remaining Frame
                    BNE        PS_ResetCounter
                    JMP        PS_NextGame          ; Last Frame
*--
PS_ResetCounter     JSR        HBResetCounter       ; Reset VBL counter
                    LDA        #$00BE               ; Wait End of SHR Screen Line #190
                    JSR        WaitForSpot                    
*--
PS_UnpackFrame      LDY        #$AAAA               ; @ of Frame Data Line
                    JSR        GetFrameBank         ; Get Frame Data Bank
                    STA        PS_FrameBank
                    LDA:       $0003,Y              ; Unpack Picture Frame
                    XBA
                    TAX                             ; X = Offset
                    LDA:       $0005,Y              ; Y = Length
                    XBA
                    TAY
                    LDA        PS_FrameBank         ; A = AA00 Bank Hosting Scene Data
                    JSR        UnpackFrame
*--
*----  Attract Message (in needed)  ----
                    LDA        PS_IsAttractMode     ; Add Attract Message at the Top of the Screen
                    BEQ        PS_IncSceneFrame
*--
PS_AMn_RemapColor_1 LDA        PS_SceneID           ; Remap Colors for Scene #0B
                    CMP        #$000B
                    BNE        PS_AMn_Display
                    LDX        #PressFireButton     ; @Data
                    LDY        #$03A8               ; Data Length
                    LDA        #$2255               ; Color 2 -> 5
                    JSL        RemapBitmapColor
                    LDX        #PressFireButton     ; @Data
                    LDY        #$03A8               ; Data Length
                    LDA        #$1144               ; Color 1 -> 4
                    JSL        RemapBitmapColor
*--
PS_AMn_Display      LDX        #PressFireButton
                    LDY        #$2160
                    LDA        #$3409
                    CLC                             ; 01/2000
                    JSL        CopyTransBitmap
*--                    
PS_AMn_RemapColor_2 LDA        PS_SceneID           ; Remap Back Colors for Scene #0B
                    CMP        #$000B
                    BNE        PS_IncSceneFrame
                    LDX        #PressFireButton     ; @Data
                    LDY        #$03A8               ; Data Length
                    LDA        #$4411               ; Color 4 -> 1
                    JSL        RemapBitmapColor 
                    LDX        #PressFireButton     ; @Data
                    LDY        #$03A8               ; Data Length
                    LDA        #$5522               ; Color 5 -> 2
                    JSL        RemapBitmapColor 
*--
PS_IncSceneFrame    INC        PS_SceneFrame        ; Current Frame # 1-N
*----
PS_DisplayDiskette  JSR        GetSaveLevel         ; Display Diskette Icon
                    BEQ        PS_DisplayEraseHint
                    LDA        PS_SceneOffset       ; 0->0, 8->1
                    LSR
                    LSR
                    LSR
                    XBA
                    ORA        PS_SceneID
                    TAX
                    JSR        DisplayFrameDisk
*----
PS_DisplayEraseHint LDA        PS_IsAttractMode     ; Display / Erase Hint on the Screen
                    BNE        PS_GetUserAction
                    JSR        GetDifficulty        
                    CMP        #$0001               ; None
                    BEQ        PS_GetUserAction
                    CMP        #$0004               ; Hard
                    BEQ        PS_GetUserAction
*--
PS_DEH_Erase        LDA        PS_FrameHasHint      ; Erase Previous Hint
                    BEQ        PS_DEH_Begin
                    STZ        PS_FrameHasHint
                    JSR        EraseFrameHint
*--                    
PS_DEH_Begin        LDA        PS_SceneFrame        ; Get Hint for this Frame
                    DEC
                    ASL
                    TAX
                    LDA        PS_HintTimeLapse,X   ; Action
                    BEQ        PS_GetUserAction     ; No Hint for this Frame
                    PHA
                    LDA        PS_SceneOffset       ; 0->0, 8->1
                    LSR
                    LSR
                    LSR
                    XBA
                    ORA        PS_SceneID           ; Display Action Hint
                    TAX                             ; SceneID|Screen#
                    PLA                             ; Action
                    JSR        DisplayFrameHint     ; Remap Color + Display Hint Bitmap
                    INC        PS_FrameHasHint      ; Hint is on Screen
*----
PS_GetUserAction    LDA        PS_SceneID           ; Get Keyboard Events
                    LDX        PS_IsAttractMode
                    JSR        GetUserAction        ; Process User Key
                    CMP        #$0000
                    BEQ        PS_CheckTimer        ; Nothing to process
                    STZ        PS_UserAction        ; Reset current User Action
                    JSR        ProcessUserAction    ; Process Pause / Audio / Save
                    CMP        #$0000      
                    BEQ        PS_CheckTimer
                    CMP        #$0051               ; Q = Quit Game
                    BEQ        PS_ActionKeyExit
                    CMP        #$0045               ; Esc = Escape to Title screen
                    BEQ        PS_ActionKeyExit                    
                    CMP        #$004C               ; L = Load Level
                    BEQ        PS_ActionKeyExit
                    CMP        #$0030               ; Action                       
                    BEQ        PS_ActionKey
                    CMP        #$0038               ; Up                       
                    BEQ        PS_DirectionKey
                    CMP        #$0034               ; Left                       
                    BEQ        PS_DirectionKey
                    CMP        #$0035               ; Down                       
                    BEQ        PS_DirectionKey
                    CMP        #$0036               ; Right                       
                    BEQ        PS_DirectionKey                                                            
                    BRA        PS_CheckTimer 
PS_ActionKeyExit    JMP        PS_Exit   
*--
PS_ActionKey        LDX        PS_IsAttractMode     ; Action Key : If Attract Mode => Start Game
                    CPX        #$0001
                    BEQ        PS_ActionKeyExit     ; Action + Attract Mode = Start Game
                    LDA        #$0030
PS_DirectionKey     STA        PS_UserAction        ; Latest User Action
*--                    
PS_CheckTimer       JSR        HBGetCounter         ; Get 1/60 sec
                    CMP        #$0007
                    BCC        PS_GetUserAction     ; Branch if < 7
*----
PS_CheckUserAction  LDA        PS_IsAttractMode     ; Attract Mode or None => Nothing to check
                    BNE        PS_NextFrame
                    JSR        GetDifficulty        ; Difficulty : 1 = None
                    CMP        #$0001
                    BEQ        PS_NextFrame
*--
PS_ExpectedAction   
* BRA        PS_NextFrame         ; DEBUG
                    LDA        PS_SceneFrame        ; Get Expected Action for this Frame
                    DEC
                    ASL
                    TAX
                    LDA        PS_ActionTimeLapse,X ; Expected Action
                    BEQ        PS_ForbiddenAction   ; Nothing
                    CMP        PS_UserAction
                    BEQ        PS_NextFrame
                    LDA        PS_DeathTimeLapse,X  ; Death ID
                    BRA        PS_Death
*--
PS_ForbiddenAction  LDA        PS_ForbiddenMoves    ; Forbidden Action #1
                    BEQ        PS_NextFrame
                    CMP        PS_UserAction
                    BNE        PS_FA_1
                    LDA        PS_ForbiddenMoves+2  ; Death ID
                    BRA        PS_Death
PS_FA_1             LDA        PS_ForbiddenMoves+4  ; Forbidden Action #2
                    BEQ        PS_NextFrame
                    CMP        PS_UserAction
                    BNE        PS_NextFrame
                    LDA        PS_ForbiddenMoves+6  ; Death ID
                    BRA        PS_Death                    
*----
PS_NextFrame        DEC        PS_LoopFrame+1       ; NbFrame--
                    LDA        PS_UnpackFrame+1     ; Next Frame Offset
                    CLC
                    ADC        #$000A               ; Data Line length
                    STA        PS_UnpackFrame+1
*--
PS_IncrementScore   LDA        PS_SceneID           ; Increment Score
                    CMP        #$0007               ; First Active Scene
                    BLT        PS_NextFrame_1
                    JSR        IncrementScore       ; Score += 100
*--
PS_NextFrame_1      JMP        PS_LoopFrame

*----  Death Sequence : Death Animation + Decrement Life + Display Life/Score Screen
PS_Death            JSR        PlayDeath            ; A = Death Scene ID
                    JSR        DecrementLife        ; Decrement Life Counter
                    JSR        ResetTimerSec        ; Start Timer
                    LDA        #$0000               
                    JSR        ScoreLifeScreen      ; Life/Score Screen
                    JSR        ClearKDBStrobe
GS_Death_3Sec       JSR        CheckUserAction      ; Check Keyboard
                    BNE        GS_Death_Exit                          
                    JSR        GetTimerSec          ; We need to wait at least 3 sec
                    CMP        #$0003
                    BMI        GS_Death_3Sec
GS_Death_Exit       LDA        #$FFFF               
                    BRA        PS_Exit              ; We have to restart the Scene
                    
*----  Jump To Next Screen (if any)  ----
PS_NextGame         LDA        PS_SceneNbScreen     ; Next Screen
                    DEC
                    BEQ        PS_End
                    STA        PS_SceneNbScreen
                    LDA        PS_SceneOffset       ; New Offset
                    CLC
                    ADC        #$0008
                    STA        PS_SceneOffset
                    JMP        PS_GameBackground
*----
PS_Exit             PHA
                    JSR        StopSound            ; Stop Sound
                    PLA                             ; Exit with Action code
                    RTS 
*----
PS_End              JSR        StopSound            ; Make Sure Sound is not playing
*----
                    JSR        GetSaveLevel         ; Save Game (if required)
                    BEQ        PS_End_1
                    LDA        PS_SceneID
                    JSR        GetNextSceneID
                    JSR        SaveGame
*----
*PS_End_1            LDA        PS_SceneAction+4     ; Debug Death Scene
*                    BEQ        PS_End_2
*                    JSR        PlayDeath            ; A = Death Scene ID    
*PS_End_2            LDA        PS_SceneAction+10
*                    BEQ        PS_End_3
*                    JSR        PlayDeath            ; A = Death Scene ID    
*PS_End_3            LDA        PS_SceneAction+16
*                    BEQ        PS_End_4
*                    JSR        PlayDeath            ; A = Death Scene ID                    
*PS_End_4            LDA        PS_SceneAction+22
*                    BEQ        PS_End_5
*                    JSR        PlayDeath            ; A = Death Scene ID                    
PS_End_1            LDA        #$0000               ; We can continue to next Scene
                    RTS
                    
*----------   PlayScene Variables   ---------
PS_SceneID          HEX        0000                 ; SceneID : 0006 - 003C
PS_IsAttractMode    HEX        0000                 ; 1 if we are displaying Attract Mode
PS_SceneFrame       HEX        0000                 ; 0000 if this is the first Frame => Add into BUFFER32KB + FadeIn
PS_FrameBank        HEX        0000                 ; Memory Bank Hosting Frame Data
PS_SceneOffset      HEX        0000
PS_FrameHasHint     HEX        0000                 ; The current Frame displays a Hint
PS_UserAction       HEX        0000                 ; Latest User Action (0=None,30=Action,38=UP,34=Left,35=Down,36=Right)
*-- All Game Screens
PS_SceneNbScreen    HEX        0000                 ; Number of Game Screen in this Scene
PS_SceneScreen      HEX        0000,0000,0000,0000  ; Game Screen DataLine Offset + Nb Game Frame + Game Frame DataLine Offset + Game Sound DataLine Offset
                    HEX        0000,0000,0000,0000  ; Game Screen DataLine Offset + Nb Game Frame + Game Frame DataLine Offset + Game Sound DataLine Offset
                    HEX        0000,0000,0000,0000  ; Game Screen DataLine Offset + Nb Game Frame + Game Frame DataLine Offset + Game Sound DataLine Offset
*-- All Scene Actions
PS_SceneNbAction    HEX        0000                 ; Number of Actions in this Scene
PS_SceneAction      HEX        0000,0000,0000       ; Frame #, Action, Death ID
                    HEX        0000,0000,0000
                    HEX        0000,0000,0000
                    HEX        0000,0000,0000
                    HEX        0000,0000,0000       ; 5 = Maximum number of action in a Scene
*-- Scene Timelapse 
PS_ActionTimeLapse  HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; 70 = Maximum number of frame in a Scene
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
PS_DeathTimeLapse   HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000                    
PS_HintTimeLapse    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000                    
*-- Forbidden Moves
PS_ForbiddenMoves   HEX        0000,0000            ; Action, Death ID (2 = Maximum Forbidden Moves per Scene)
                    HEX        0000,0000            ; Action, Death ID


******************************************************************
*  PlayDeath() :  Run a Death Animation + Sound (IN:A:=SceneID). *
******************************************************************
PlayDeath           STA        PD_SceneID           ; Scene ID
                    LDX        #PD_SceneScreen
                    JSR        GetDeathInfo         ; Get Death Info
   
*----  Death Background : Decompress Death Screen Pixels  ----
PD_DeathBackground  JSR        FadeOut              ; FadeOut to Black
*--
                    LDX        PD_SceneOffset
                    LDA        PD_SceneScreen+0,X   ; Death Screen Offset
                    TAY
*--
                    JSR        GetFrameBank         ; Get Frame Data Bank
                    STA        PD_FrameBank
                    LDA:       $0003,Y              ; X = Offset
                    XBA
                    TAX
                    LDA:       $0005,Y              ; Y = Length
                    XBA
                    TAY
                    LDA        PD_FrameBank         ; A = AA00 Bank Hosting Frame Data
                    ORA        CurrentBank          ; 00BB               
                    JSR        UnpackLZ4            ; A = BankSrc,BankDst / X = 1st Packed Byte offset / Y = Pack Data Size
                    JSR        CopyPixelSCB         ; Copy Pixels + SCB to $01/2000

*----  Death Frame #0 : Add Frame #0 Pixel Data  ----
PD_DeathFrame0      LDX        PD_SceneOffset       ; Display Frame #0 Bitmap
                    LDA        PD_SceneScreen+4,X   ; Frame #0 Offset
                    TAY
PD_UnpackFrame0     JSR        GetFrameBank         ; Get Frame Data Bank
                    STA        PD_FrameBank
                    LDA:       $0003,Y              ; Unpack Picture Frame
                    XBA
                    TAX                             ; X = Offset
                    LDA:       $0005,Y              ; Y = Length
                    XBA
                    TAY
                    LDA        PD_FrameBank         ; A = AA00 Bank Hosting Scene Data
                    JSR        UnpackFrame

*----  Fade In  ----
PD_FadeIn           LDA        #BUFFER32KB          ; FadeIn to Death Screen
                    LDY        #$FFFF               ; Don't Copy Pixel, Only Palette
                    JSR        FadeIN

*----  Play Sound (if there)  ----                    
PD_DeathSound       LDX        PD_SceneOffset
                    LDA        PD_SceneScreen+6,X   ; Death Sound Offset
                    BEQ        PD_DeathScene1N      ; No Sound
                    TAY
*--
PD_PlayDeathSound   JSR        GetFrameBank         ; Get Frame Data Bank
                    STA        PD_FrameBank                    
                    LDA:       $0003,Y              ; X = Sound Address
                    XBA
                    TAX
                    LDA:       $0005,Y              ; Y = Sound Length
                    XBA
                    TAY
                    LDA        PD_FrameBank         ; A = AA00 Bank Hosting Frame Data
                    JSR        CopySoundDataToRAM   ; Copy Sound Data to Ram Sound
                    LDA        PD_SceneID           ; Scene ID
                    ORA        #$8000
                    JSR        PlaySoundData        ; Start playing Sound

*----  Play Death Frames 1-N  ----
PD_DeathScene1N     LDX        PD_SceneOffset       ; Init Display Frame #1-N
                    LDA        PD_SceneScreen+2,X   ; Number of Death Frame
                    STA        PD_LoopFrame+1
                    LDA        PD_SceneScreen+4,X   ; Frame #N Offset
                    STA        PD_UnpackFrame+1
*--
PD_InitTimer        JSR        HBResetCounter       ; Reset VBL counter
                    BRA        PD_CheckTimer        ; The first frame is already on the screen, so wait
*----
PD_LoopFrame        LDA        #$AAAA               ; Number of remaining Frame
                    BEQ        PD_End               ; Last Frame
*--
PD_ResetCounter     JSR        HBResetCounter       ; Reset VBL counter
                    LDA        #$00BE               ; Wait End of SHR Screen Line #190
                    JSR        WaitForSpot                    
*--
PD_UnpackFrame      LDY        #$AAAA               ; @ of Frame Data Line
                    JSR        GetFrameBank         ; Get Frame Data Bank
                    STA        PD_FrameBank
                    LDA:       $0003,Y              ; Unpack Picture Frame
                    XBA
                    TAX                             ; X = Offset
                    LDA:       $0005,Y              ; Y = Length
                    XBA
                    TAY
                    LDA        PD_FrameBank         ; A = AA00 Bank Hosting Scene Data
                    JSR        UnpackFrame
*----                  
PD_CheckTimer       JSR        HBGetCounter         ; Get 1/60 sec
                    CMP        #$0007
                    BCC        PD_CheckTimer        ; Branch if < 7
*----
PD_NextFrame        DEC        PD_LoopFrame+1       ; NbFrame--
                    LDA        PD_UnpackFrame+1     ; Next Frame Offset
                    CLC
                    ADC        #$000A               ; Data Line Length
                    STA        PD_UnpackFrame+1
PD_NextFrame_1      BRA        PD_LoopFrame
*----
PD_End              JSR        StopSound            ; Make Sure Sound is not playing
*--
                    RTS

*----------   PlayDeath Variables   ---------
PD_SceneID          HEX        0000
PD_SceneOffset      HEX        0000
PD_FrameBank        HEX        0000                 ; Memory Bank Hosting Frame Data
PD_SceneScreen      HEX        0000,0000,0000,0000  ; Death Screen Offset + Nb Death Frame + Death Frame Offset + Death Sound Offset


**********************************************************************************************************
*  GetSceneInfo() :  Get Information about Scene Animation and Sound (IN:A=SceneNumber,IN:X=@SceneInfo). *
**********************************************************************************************************
GetSceneInfo        STA        GSI_2+1              ; Scene Number : 0006 - 003E
                    STA        GSI_3+1        
                    STX        GSI_Copy_2+1
                    LDX        #$0018
GSI_Reset           STZ        GSI_NbScreen,X
                    DEX
                    DEX
                    BPL        GSI_Reset  
*--
                    LDX        #$FFFE               ; Search Scene #ID : 0 - N
GSI_1               INX
                    INX
                    LDA        TAB_SceneID,X        ; Scene ID Table
GSI_2               CMP        #$AAAA
                    BNE        GSI_1
                    LDA        TAB_SceneAddr,X      ; @ of Scene 
                    STA        GSI_Begin+1
*--
GSI_Begin           LDA        #$AAAA               ; Disk(1)Data(2)Offset(2)Length(2)Scene(1)Type(1)Frame(1)          
                    TAY                             ; Y = @ Line Frame Data Information
                    CLC
                    ADC        #$000A               ; Next Line = @Line + 10
                    STA        GSI_Begin+1
*--
                    LDA:       $0007,Y              ; Scene Number
                    AND        #$00FF
GSI_3               CMP        #$AAAA               ; Out of the Scene => Exit
                    BEQ        GSI_4
                    JMP        GSI_End
GSI_4               LDA:       $0008,Y              ; Type : GameBackground / GameFrame / GameSound / DeathBackground / DeathFrame / DeathSound 
                    AND        #$00FF
                    CMP        #$0000               ; 00=GameBackground
                    BEQ        GSI_GScreen
                    CMP        #$0001               ; 01=GameFrame
                    BEQ        GSI_GFrame
                    CMP        #$0002               ; 02=GameSound
                    BEQ        GSI_GSound
                    BRA        GSI_Begin
*--                    
GSI_GScreen         INC        GSI_NbScreen         ; Game Screen Background
                    LDA        GSI_NbScreen
                    DEC
                    ASL
                    TAX
                    LDA        GSI_TabScreen,X
                    TAX
                    TYA
                    STA:       $0000,X              ; Game  Screen Offset
                    JMP        GSI_Begin
*--
GSI_GFrame          LDA        GSI_NbScreen         ; Game Animation Frame
                    DEC
                    ASL
                    TAX
                    LDA        GSI_TabScreen,X
                    TAX
                    LDA:       $0002,X              ; Number of Game Animation Frame
                    INC
                    STA:       $0002,X
                    CMP        #$0001
                    BNE        GSI_GFrame_1
                    TYA
                    STA:       $0004,X              ; Game Frame Offset
GSI_GFrame_1        JMP        GSI_Begin
*--
GSI_GSound          LDA        GSI_NbScreen         ; Game Sound
                    DEC
                    ASL
                    TAX
                    LDA        GSI_TabScreen,X
                    TAX
                    TYA
                    STA:       $0006,X              ; Game Sound Offset
                    JMP        GSI_Begin
*--
GSI_End             LDX        #$0018               ; Copy Back Scene Information
GSI_Copy_1          LDA        GSI_NbScreen,X
GSI_Copy_2          STA        $AAAA,X
                    DEX
                    DEX
                    BPL        GSI_Copy_1  
                    RTS

*----------   GetSceneInfo Variables   ---------

GSI_TabScreen       DA         GSI_Screen1,GSI_Screen2,GSI_Screen3
GSI_NbScreen        HEX        0000                 ; Number of Game Screen in this Scene
GSI_Screen1         HEX        0000,0000,0000,0000  ; Game Screen Offset + Nb Game Frame + Game Frame Offset + Game Sound Offset
GSI_Screen2         HEX        0000,0000,0000,0000  ; Game Screen Offset + Nb Game Frame + Game Frame Offset + Game Sound Offset
GSI_Screen3         HEX        0000,0000,0000,0000  ; Game Screen Offset + Nb Game Frame + Game Frame Offset + Game Sound Offset


**********************************************************************************************************
*  GetDeathInfo() :  Get Information about Death Animation and Sound (IN:A=SceneNumber,IN:X=@SceneInfo). *
**********************************************************************************************************
GetDeathInfo        STA        GDI_2+1              ; Scene Number : 0006 - 0030
                    STA        GDI_3+1        
                    STX        GDI_Copy_2+1
                    LDX        #$0006
GDI_Reset           STZ        GDI_Screen,X
                    DEX
                    DEX
                    BPL        GDI_Reset  
*--
                    LDX        #$FFFE               ; Search Scene #ID : 0 - N
GDI_1               INX
                    INX
                    LDA        TAB_SceneID,X        ; Scene ID Table
GDI_2               CMP        #$AAAA
                    BNE        GDI_1
                    LDA        TAB_SceneAddr,X      ; @ of Scene 
                    STA        GDI_Begin+1
*--
GDI_Begin           LDA        #$AAAA               ; Disk(1)Data(2)Offset(2)Length(2)Scene(1)Type(1)Frame(1)          
                    TAY                             ; Y = @ Line Frame Data Information
                    CLC
                    ADC        #$000A               ; Next Line = @Line + 10
                    STA        GDI_Begin+1
*--
                    LDA:       $0007,Y              ; Scene Number
                    AND        #$00FF
GDI_3               CMP        #$AAAA               ; Out of the Scene => Exit
                    BNE        GDI_End
GDI_4               LDA:       $0008,Y              ; Type : GameBackground / GameFrame / GameSound / DeathBackground / DeathFrame / DeathSound 
                    AND        #$00FF
                    CMP        #$0080               ; 80=DeathBackground
                    BEQ        GDI_DScreen
                    CMP        #$0081               ; 81=DeathFrame
                    BEQ        GDI_DFrame
                    CMP        #$0082               ; 82=DeathSound
                    BEQ        GDI_DSound
                    BRA        GDI_Begin
*--
GDI_DScreen         TYA                             ; Death Background Screen
                    STA        GDI_Screen+0         ; Death Screen Offset
                    BRA        GDI_Begin
*--
GDI_DFrame          LDA        GDI_Screen+2         ; Death Animation Frame
                    INC
                    STA        GDI_Screen+2
                    CMP        #$0001
                    BNE        GDI_DFrame_1
                    TYA
                    STA        GDI_Screen+4         ; Death Frame Offset
GDI_DFrame_1        BRA        GDI_Begin
*--
GDI_DSound          TYA                             ; Death Sound
                    STA:       GDI_Screen+6         ; Death Sound Offset
                    BRA        GDI_Begin
*--
GDI_End             LDX        #$0006               ; Copy Back Death Information
GDI_Copy_1          LDA        GDI_Screen,X
GDI_Copy_2          STA        $AAAA,X
                    DEX
                    DEX
                    BPL        GDI_Copy_1  
                    RTS

*----------   GetDeathInfo Variables   ---------

GDI_Screen          HEX        0000,0000,0000,0000  ; Death Screen Offset + Nb Death Frame + Death Frame Offset + Death Sound Offset


******************************************************************************************
*  GetSceneAction() :  Get Actions used in a Scene (IN:A=SceneNumber,IN:X=@SceneAction). *
******************************************************************************************
GetSceneAction      STA        GSA_2+1              ; Scene Number : 0006 - 0030C    
                    STX        GSA_Reset+1
                    STX        GSA_NbAction+1
                    INX
                    INX
                    STX        GSA_Frame_W+1
                    STX        GSA_Action_W+1
                    STX        GSA_Death_W+1
*--                    
                    LDX        #$001E               ; Reset Number of Actions + 5 Actions
GSA_Reset           STZ        $CCCC,X
                    DEX
                    DEX
                    BPL        GSA_Reset
*--
                    LDX        #$FFFE               ; Search Scene #ID : 0 - N
GSA_1               INX
                    INX
                    LDA        TAB_SceneID,X        ; Scene ID Table
GSA_2               CMP        #$AAAA               ; Scene ID
                    BNE        GSA_1
                    LDA        TAB_SceneAction,X    ; @ of Scene Action
                    STA        GSA_Begin+1
                    INC
                    STA        GSA_Frame_R+1
                    STA        GSA_Action_R+1
                    STA        GSA_Death_R+1
*--
GSA_Begin           LDA        $AAAA                ; Number of Actions
                    AND        #$00FF
                    BEQ        GSA_End
                    STA        GSA_Begin_1+1        ; *3
                    ASL
                    CLC
GSA_Begin_1         ADC        #$AAAA
                    STA        GSA_Loop+1
*--
                    LDX        #$0000
                    LDY        #$0000
GSA_Frame_R         LDA        $AAAA,X              ; Frame #
                    AND        #$00FF
GSA_Frame_W         STA        $BBBB,Y               
                    INX
                    INY
                    INY
GSA_Action_R        LDA        $AAAA,X              ; Action (Action=80,Down=08,Up=04,Right=02,Left=01)
                    AND        #$00FF
                    CMP        #$0080               ; Convert Action Code (80,08,04,02,01)
                    BEQ        GSA_Action_Action    ; into User Action    (30,35,38,36,31)
                    CMP        #$0008
                    BEQ        GSA_Action_Down
                    CMP        #$0004
                    BEQ        GSA_Action_Up
                    CMP        #$0002
                    BEQ        GSA_Action_Right
                    CMP        #$0001
                    BEQ        GSA_Action_Left
                    BRA        GSA_Action_W         ; No Translation
GSA_Action_Action   LDA        #$0030
                    BRA        GSA_Action_W
GSA_Action_Up       LDA        #$0038
                    BRA        GSA_Action_W
GSA_Action_Down     LDA        #$0035
                    BRA        GSA_Action_W
GSA_Action_Left     LDA        #$0034
                    BRA        GSA_Action_W
GSA_Action_Right    LDA        #$0036
GSA_Action_W        STA        $BBBB,Y 
                    INX              
                    INY
                    INY
GSA_Death_R         LDA        $AAAA,X              ; Death ID
                    AND        #$00FF 
GSA_Death_W         STA        $BBBB,Y              
                    INX
                    INY
                    INY
GSA_NbAction        INC        $CCCC                ; One more action
GSA_Loop            CPX        #$AAAA               ; NbAction * 3
                    BNE        GSA_Frame_R       
*--
GSA_End             RTS


******************************************************************************************
*  DisplayFrameHint() :  Display Hint bitmap 12x12 on Screen (IN:A=Action,IN:X=SceneID). *
******************************************************************************************
DisplayFrameHint    STX        DFH_CopyBitmap+1     ; Scene ID
                    CMP        #$0034               ; Left
                    BEQ        DFH_Left
                    CMP        #$0036               ; Right
                    BEQ        DFH_Right
                    CMP        #$0038               ; Up
                    BEQ        DFH_Up
                    CMP        #$0035               ; Down
                    BEQ        DFH_Down
                    BRA        DFH_Action           ; Action
*--
DFH_Left            LDX        #Aplha_Left          ; <-
                    BRA        DFH_Event
DFH_Right           LDX        #Aplha_Right         ; ->
                    BRA        DFH_Event
DFH_Down            LDX        #Aplha_Down          ; v
                    BRA        DFH_Event
DFH_Up              LDX        #Aplha_Up            ; ^
                    BRA        DFH_Event
DFH_Action          LDX        #Aplha_Action        ; Action
*--
DFH_Event           JSR        GetDifficulty        ; Difficulty : 2 = Easy / 3 = Medium
                    CMP        #$0002
                    BEQ        DFH_CopyBitmap
                    LDX        #Aplha_Event         ; Event
*--
DFH_CopyBitmap      LDA        #$AAAA               ; Scene ID
                    JSL        DisplayHintBitmap
*--                       
                    RTS


*************************************************************
*  EraseFrameHint() :  Erase Hint bitmap 12x12 from Screen. *
*************************************************************
EraseFrameHint      LDY        #$000B
                    LDX        #$220D               ; Top Center of the Screen
EFH_0               LDAL       $012000,X
                    STAL       $E12000,X
                    LDAL       $012002,X
                    STAL       $E12002,X
                    LDAL       $012004,X
                    STAL       $E12004,X
                    TXA                             ; Next Line
                    CLC
                    ADC        #$00A0
                    TAX
                    DEY
                    BPL        EFH_0
                    RTS


**********************************************************************************
*  DisplayFrameDisk() :  Display Diskette bitmap 12x12 on Screen (IN:X=SceneID). *
**********************************************************************************
DisplayFrameDisk    TXA                             ; Scene ID
                    LDX        #Aplha_Diskette      ; Diskette Bitmap
                    JSL        DisplayDiskBitmap
                    RTS


********************************************************************************
*  GetNextSceneID() :  Compute next Scene ID (IN:A=SceneID,OUT:A=NextSceneID). *
********************************************************************************
GetNextSceneID      STA        GNSID_2+1
                    LDX        #$0000
GNSID_1             LDA        GS_PlayScene_Tab,X
GNSID_2             CMP        #$AAAA               ; Current Scene ID 
                    BEQ        GNSID_3
                    INX
                    INX
                    BRA        GNSID_1
GNSID_3             LDA        GS_PlayScene_Tab+2,X
                    RTS
                    

************************************************************************************************
*  UnpackFrame() :  Uncompress one Frame of the animation (IN:A=Bank,IN:X=Offset,IN:Y=Length). *
************************************************************************************************
UnpackFrame         CPY        #$0002               ; Scene Data Length
                    BNE        UnpackFrame_Init
                    JMP        UF_End               ; Empty Frame
*--------
UnpackFrame_Init    STA        UF_FirstJump_1+2     ; Scene Data Bank  BB00
                    STA        UF_LoadData+2
                    STA        UF_JumpAbsolute+2
                    ORA        #$0001               ; Scene Data Bank -> $01/2000
                    STA        UF_NewData_MVM+1        
                    LDA        #$0100               ; Current Bank -> $01/2000
                    ORA        CurrentBank          ;   00YY     
                    XBA               
                    STA        UF_Color_MVM+1
                    STA        UF_Background_MVM+1
                    STX        UF_FirstJump+1       ; Src Scene Data Offset
                    TYA                             ; Scene Data Length
                    CLC
                    ADC        UF_FirstJump+1       ; Src Scene Data Offset
                    STA        UF_NextItem+1        ; Scene Data End
*--------
UF_FirstJump        LDX        #$AAAA               ; Src : Scene Data Offset
UF_FirstJump_1      LDAL       $BB0000,X            ; The first 2 bytes is an Absolute Jump     
                    CLC
                    ADC        #$2000
                    TAY                             ; Dst : Screen Offset   $01/2000
                    INX
                    INX        
*--------      
UF_LoadData         LDAL       $BB0000,X            ; Get Data Control Byte
                    STA        UF_DataControl                    
                    AND        #$00FF
                    BNE        UF_LoadData_1
                    INX                             ; 00.00_0000 = JMP Rel Long or JMP Abs
                    LDA        UF_DataControl+1
                    AND        #$0080
                    BNE        UF_JumpRelLong       ; 00.00_0000 1LLL_LLLL             7 bit Relative Length
                    JMP        UF_JumpAbsolute      ; 00.00_0000 0LLL_LLLL LLLL_LLLL  15 bit Absolute Address        
UF_LoadData_1       AND        #$00C0
                    BEQ        UF_Color             ; 00.LL_LCCC = COLOR
                    CMP        #$0040               ; 01.LL_LLLL = BACKGROUND
                    BEQ        UF_Background
                    CMP        #$0080               ; 10.LL_LLLL = NEW DATA
                    BEQ        UF_NewData
*--------
UF_JumpRelShort     STY        UF_JumpRelShort_1+1  ; 11.LL_LLLL = JUMP RELATIVE SHORT
                    LDA        UF_DataControl
                    AND        #$003F
                    INC
                    CLC
UF_JumpRelShort_1   ADC        #$AAAA
                    TAY                             ; Y += (LLLLLL + 1)
                    INX
                    JMP        UF_NextItem
*--------
UF_Background       PHX                             ; Inject LLL bytes from Background Picture into Steam
*--
UF_Background_Src   TYA                             ; @ of the Background Data in the Curent Bank
                    SEC
                    SBC        #$2000
                    CLC
                    ADC        #BUFFER32KB
                    TAX                             ; Background Offset
*--
UF_Background_Len   LDA        UF_DataControl       ; Compute Background Length
                    AND        #$003F               ; -1 is not needed because we store 0-63 for 1-64
*--           
UF_Background_MVM   MVN        $AA,$01              ; Current Bank -> $012000
                    PHK
                    PLB
*--
                    PLX
                    INX
                    BRA        UF_NextItem
*--------
UF_NewData          LDA        UF_DataControl       ; Inject LLL bytes of NewData into Steam
                    AND        #$003F               ; NewData Length  : -1 is not needed because we store 0-63 for of 1-64
*--
                    INX                             ; NewData Offset
*--
UF_NewData_MVM      MVN        $AA,$01              ; Scene Data Bank -> $012000
                    PHK
                    PLB
*--
                    BRA        UF_NextItem
*--------
UF_Color            PHX                             ; Inject LLL bytes of CCC color into Steam
*--  
UF_Color_Color      LDA        UF_DataControl       ; Color Byte
                    AND        #$0007
                    ASL
                    TAX
                    LDA        UF_ColorColorTab,X   ; Color 01-07 Table
                    STA        UF_Color_StoreColor+1
*--
UF_Color_Len        LDA        UF_DataControl       ; Compute Color Length
                    LSR
                    LSR
                    LSR
                    AND        #$0007
                    TAX
                    LDA        UF_ColorLength,X     ; 1, 2, 3, 6, 10, 20, 30, 40
                    AND        #$00FF
                    DEC                             ; Color Length - 1
*--
UF_Color_StoreColor LDX        #$AAAA               ; Src Color
UF_Color_MVM        MVN        $AA,$01              ; Current Bank -> $012000
                    PHK
                    PLB
*--       
                    PLX
                    INX
                    BRA        UF_NextItem
*--------
UF_JumpRelLong      STY        UF_JumpRelLong_1+1   ; 0000_0000 1LLL_LLLL  Jump Relative Long
                    LDA        UF_DataControl+1
                    AND        #$007F               ; Reset Bit #7
                    CLC
                    ADC        #$0041               ; + 1 + 64
UF_JumpRelLong_1    ADC        #$AAAA               ; Y += Length
                    TAY
                    INX
                    BRA        UF_NextItem
*--------
UF_JumpAbsolute     LDAL       $BB0000,X            ; 0000_0000 0@@@_@@@@ @@@@_@@@@ Jmp Absolute
                    XBA                             ; Stored in inverse order to expose Bit #15 to 0
                    CLC
                    ADC        #$2000               ; Y = @ + $2000
                    TAY
                    INX
                    INX
*--------
UF_NextItem         CPX        #$AAAA               ; Check End of Frame Data
                    BEQ        UF_End                    
                    JMP        UF_LoadData
*--
UF_End              RTS       

*----------   UnpackFrame Variables   ---------
UF_DataControl      HEX        0000
UF_DataLength       HEX        0000
UF_ColorLength      HEX        01,02,03,06,0A,14,1E,28
UF_ColorColorTab    DA         UF_ColorColor0,UF_ColorColor1,UF_ColorColor2,UF_ColorColor3,UF_ColorColor4,UF_ColorColor5,UF_ColorColor6,UF_ColorColor7
UF_ColorColor0                                      ; Color 0 is never used
UF_ColorColor1      HEX        11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11
UF_ColorColor2      HEX        22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
UF_ColorColor3      HEX        33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33
UF_ColorColor4      HEX        44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44
UF_ColorColor5      HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
UF_ColorColor6      HEX        66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66
UF_ColorColor7      HEX        77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77


********************************************************************************************
*  GetUserAction() :  Read Keyboard Inputs (IN:A=SceneID,IN:X=IsAttractMode,OUT:A=Action). *
********************************************************************************************
GetUserAction       STA        GUA_SceneID          ; SceneID
                    STX        GUA_IsAttractMode    ; Attract Mode
                    LDA        #$0000               ; Default Action = None
                    SEP        #$20
                    LDAL       KBDREAD
                    BMI        GUA_Decode
                    JMP        GUA_Empty
*----
GUA_Decode          AND        #$7F                 ; Clear Bit #7
                    CMP        #$0D                 ; Return
                    BEQ        GUA_Action
                    CMP        #$20                 ; Space
                    BEQ        GUA_Action
                    CMP        #$30                 ; 0
                    BEQ        GUA_Action
*--
                    CMP        #$0B                 ; Up
                    BEQ        GUA_Up
                    CMP        #$38                 ; 8
                    BEQ        GUA_Up
*--
                    CMP        #$0A                 ; Down
                    BEQ        GUA_Down
                    CMP        #$35                 ; 5
                    BEQ        GUA_Down
*--
                    CMP        #$15                 ; Right
                    BEQ        GUA_Right
                    CMP        #$36                 ; 6
                    BEQ        GUA_Right
*--
                    CMP        #$08                 ; Left
                    BEQ        GUA_Left
                    CMP        #$34                 ; 4
                    BEQ        GUA_Left
*--
                    CMP        #$1B                 ; Esc
                    BEQ        GUA_Esc
*--
                    CMP        #$51                 ; Q
                    BEQ        GUA_Quit
                    CMP        #$71                 ; q
                    BEQ        GUA_Quit
*--
                    CMP        #$4C                 ; L
                    BEQ        GUA_LoadLevel
                    CMP        #$6C                 ; l
                    BEQ        GUA_LoadLevel
*--
                    CMP        #$53                 ; S
                    BEQ        GUA_SaveLevel                    
                    CMP        #$73                 ; s
                    BEQ        GUA_SaveLevel    
*--
                    CMP        #$50                 ; P
                    BEQ        GUA_Pause                               
                    CMP        #$70                 ; p
                    BEQ        GUA_Pause   
*--
                    CMP        #$41                 ; A
                    BEQ        GUA_Audio                       
                    CMP        #$61                 ; a
                    BEQ        GUA_Audio   
*--
                    BRA        GUA_Empty             ; Other = Ignore
*----
GUA_Action          LDA        #$30                 ; 0
                    BRA        GUA_Validate
GUA_Up              LDA        #$38                 ; 8
                    BRA        GUA_Validate
GUA_Down            LDA        #$35                 ; 5
                    BRA        GUA_Validate
GUA_Right           LDA        #$36                 ; 6
                    BRA        GUA_Validate
GUA_Left            LDA        #$34                 ; 4
                    BRA        GUA_Validate
GUA_Esc             LDA        #$45                 ; E
                    BRA        GUA_Validate
GUA_Quit            LDAL       BUTN0                ; Open Apple Key (Bit #7)
                    BPL        GUA_Empty
                    LDA        #$51                 ; Q
                    BRA        GUA_Validate
GUA_LoadLevel       LDA        #$4C                 ; L
                    BRA        GUA_Validate
GUA_SaveLevel       LDA        #$53                 ; S
                    BRA        GUA_Validate
GUA_Pause           LDA        #$50                 ; P
                    BRA        GUA_Validate
GUA_Audio           LDA        #$41                 ; A
                    BRA        GUA_Validate
*--
GUA_Empty           LDA        #$00                 ; No Action
*----
GUA_Validate        STAL       KBDSTRB              ; Reset Keyboard Input
                    REP        #$30
                    AND        #$00FF
*--
GUA_Validate_Save   CMP        #$0053               ; Validate 'Save Game' action
                    BNE        GUA_End
                    JSR        GetDifficulty        ; No Save on Easy Mode
                    CMP        #$0001
                    BEQ        GUA_Validate_End
                    LDA        GUA_IsAttractMode    ; No Save on Attract Mode
                    BNE        GUA_Validate_End
                    LDA        GUA_SceneID          ; We do not allow Save
                    CMP        #$0009               ; for Scene 0009
                    BEQ        GUA_Validate_End
                    CMP        #$000B               ; for Scene 000B
                    BEQ        GUA_Validate_End
                    CMP        #$003E               ; for Scene 003E
                    BEQ        GUA_Validate_End
                    CMP        #$003C               ; for Scene 003C (The End)
                    BEQ        GUA_Validate_End
                    LDA        #$0053               ; We can Save
                    BRA        GUA_End
GUA_Validate_End    LDA        #$0000               ; No Action 
*----
GUA_End             RTS

*----------   UnpackFrame Variables   ---------
GUA_SceneID         HEX        0000
GUA_IsAttractMode   HEX        0000

*************************************************************************
*  CheckUserAction() :  Check if a key has been pressed (OUT:A=Action). *
*************************************************************************
CheckUserAction     LDA        #$0000               ; Reset 
                    SEP        #$20                 ; Check for Key Pressed 
                    LDAL       KBDREAD
                    BPL        CUA_NoKeyEnd
                    STAL       KBDSTRB
                    AND        #$7F                 ; Clear Bit #7
*--
                    CMP        #$0D                 ; Return
                    BEQ        CUA_Action
                    CMP        #$20                 ; Space
                    BEQ        CUA_Action
                    CMP        #$30                 ; 0
                    BEQ        CUA_Action
*--
                    CMP        #$1B                 ; Esc
                    BEQ        CUA_Esc
*--
                    CMP        #$51                 ; Q
                    BEQ        CUA_Quit
                    CMP        #$71                 ; q
                    BEQ        CUA_Quit
*--
                    CMP        #$4C                 ; L
                    BEQ        CUA_LoadLevel
                    CMP        #$6C                 ; l
                    BEQ        CUA_LoadLevel
*--
                    CMP        #$41                 ; A
                    BEQ        CUA_Audio                       
                    CMP        #$61                 ; a
                    BEQ        CUA_Audio
*--
                    CMP        #$50                 ; P
                    BEQ        CUA_Pause                               
                    CMP        #$70                 ; p
                    BEQ        CUA_Pause                    
*----
CUA_Action          LDA        #$30                 ; 0
                    BRA        CUA_End
CUA_Esc             LDA        #$45                 ; E
                    BRA        CUA_End
CUA_Quit            LDAL       BUTN0                ; Open Apple Key (Bit #7)
                    BPL        CUA_NoKeyEnd
                    LDA        #$51                 ; Q
                    BRA        CUA_End
CUA_LoadLevel       LDA        #$4C                 ; L
                    BRA        CUA_End
CUA_Audio           JSR        EnableDisableAudio   ; Enable / Disable Audio
                    BRA        CUA_NoKeyEnd
CUA_Pause           JSR        PauseGame            ; Pause the Game
                    BRA        CUA_NoKeyEnd
*----
CUA_End             REP        #$30                 ; Send Action Code
                    AND        #$00FF
                    RTS
*--
CUA_NoKeyEnd        REP        #$30                 ; No Key detected
                    LDA        #$0000
                    RTS


************************************************************************************
*  ProcessUserAction() :  Process User Action (IN:A=ActionCode,OUT:A=SceneAction). *
************************************************************************************
ProcessUserAction   CMP        #$0041
                    BEQ        PUA_Audio            ; Audio
                    CMP        #$0050
                    BEQ        PUA_Pause            ; Pause
                    CMP        #$0053
                    BEQ        PUA_SaveGame         ; Save
                    JMP        PUA_End              ; Not Processed
*--
PUA_Audio           JSR        MuteUnmuteAudio      ; On/Off Audio
                    LDA        #$0000
                    BRA        PUA_End
*--
PUA_Pause           JSR        PauseGame            ; Pause Game
                    LDA        #$0000
                    BRA        PUA_End
*--
PUA_SaveGame        LDA        #$0001               ; Ask for Save at the end of the Scene
                    JSR        SetSaveLevel
                    LDA        #$0000
*--
PUA_End             RTS


*******************************************************************************************
*  LoadGame() :  Load previously saved game information Scene,Life,Score (OUT:A=SceneID). *
*******************************************************************************************
LoadGame            LDA        CurrentBank          ; 00XX 
                    STA        LG_gsosREAD+6
                    LDA        #SaveGameBuffer
                    STA        LG_gsosREAD+4
*--
                    JSR        CheckSaveDisk        ; Make sure Disk #1 is available
                    BCS        LG_ErrorEnd
*--
                    JSL        GSOS                 ; Open File
                    dw         $2010                ; Code for command 'Open'
                    adrl       LG_gsosOPEN          ; Parameters Table Address (Open in Read Mode)
                    BCS        LG_ErrorEnd          ; Error
                    LDA        LG_gsosOPEN+2
                    STA        LG_gsosGETEOF+2
                    STA        LG_gsosREAD+2
                    STA        LG_gsosCLOSE+2
*--
                    JSL        GSOS                 ; Get File Size
                    dw         $2019                ; Code for command 'GetFileSize'
                    adrl       LG_gsosGETEOF        ; Parameters Table Address
                    LDA        LG_gsosGETEOF+4
                    STA        LG_gsosREAD+8        ; File Size : Should be 6 bytes long
                    CMP        #$0006
                    BNE        LG_Error
                    STZ        LG_gsosREAD+10
*--
                    JSL        GSOS                 ; Read File Content
                    dw         $2012                ; Code for command 'Read'
                    adrl       LG_gsosREAD          ; Parameters Table Address
                    BCS        LG_Error
*--
                    JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       LG_gsosCLOSE         ; Parameters Table Address
                    BRA        LG_CheckValues
*--
LG_Error            JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       LG_gsosCLOSE         ; Parameters Table Address
LG_ErrorEnd         BRA        LG_RAZ
*------
LG_CheckValues      LDA        SaveGameBuffer+0     ; Check Values
                    CMP        #$0006
                    BMI        LG_RAZ
                    LDX        #$0000               ; Check Scene ID
LG_CheckValues_1    LDA        TAB_SceneID,X
                    BEQ        LG_RAZ               ; End of Table
                    CMP        SaveGameBuffer+0
                    BEQ        LG_CheckValues_2
                    INX
                    INX
                    BRA        LG_CheckValues_1
*--
LG_CheckValues_2    LDA        SaveGameBuffer+3     ; Life
                    AND        #$00FF
                    BEQ        LG_RAZ
                    CMP        #$0004               ; Should be 1 - 3
                    BPL        LG_RAZ
*--
                    LDA        SaveGameBuffer+2     ; Score 1
                    AND        #$00FF
                    CMP        #$000A               ; 0-9
                    BPL        LG_RAZ
                    LDA        SaveGameBuffer+5     ; Score 2
                    AND        #$00FF
                    CMP        #$000A               ; 0-9
                    BPL        LG_RAZ
                    LDA        SaveGameBuffer+4     ; Score 3
                    AND        #$00FF
                    CMP        #$000A               ; 0-9
                    BPL        LG_RAZ
*------
LG_SetValues        LDX        SaveGameBuffer+2     ; Life|Score1
                    LDY        SaveGameBuffer+4     ; Score2|Score3
                    JSR        SetScoreLife
                    LDA        SaveGameBuffer+0     ; Scene ID
                    BRA        LG_End
*------
LG_RAZ              JSR        ResetScoreLife       ; Init Level + Score + Life
                    LDA        #$000B               ; Start Game at Scene #0B
*------
LG_End              RTS


*******************************************************************************************
*  SaveGame() :  Save current game information (Scene,Life,Score) to disk (IN:A=SceneID). *
*******************************************************************************************
SaveGame            STA        SaveGameBuffer+0     ; Scene ID
                    JSR        GetScoreLife
                    STX        SaveGameBuffer+2     ; Life|Score1
                    STY        SaveGameBuffer+4     ; Score2|Score3
*--
                    JSR        CheckSaveDisk        ; Make sure Disk #1 is available
                    BCS        SG_End
*--
SG_CreateEmpty      JSL        GSOS                 ; Delete File
                    dw         $2002                ; Code for command 'Destroy'
                    adrl       SG_gsosDESTROY       ; Parameters Table Address   
*--                    
                    SEP        #$20
                    LDA        #$E3
                    STA        SG_gsosCREATE+6      ; File Access
                    LDA        #$5D
                    STA        SG_gsosCREATE+8      ; File Type (BD save game)
                    LDA        #$01
                    STA        SG_gsosCREATE+14     ; Storage Type (Standard)
                    REP        #$30
                    LDA        #$801C
                    STA        SG_gsosCREATE+10     ; File AuxType (BD save game)
                    STZ        SG_gsosCREATE+12
*--
                    JSL        GSOS                 ; Create File
                    dw         $2001                ; Code for command 'Create'
                    adrl       SG_gsosCREATE        ; Parameters Table Address                    
*----
SG_WriteData        LDA        #$0002               ; Open access Mode = Write
                    STA        SG_gsosOPEN+8     
                    JSL        GSOS                 ; Open File
                    dw         $2010                ; Code for command 'Open'
                    adrl       SG_gsosOPEN          ; Parameters Table Address (Open in Read Mode)
                    BCS        SG_End               ; Error
                    LDA        SG_gsosOPEN+2        ; File Descriptor
                    STA        SG_gsosSETMARK+2
                    STA        SG_gsosWRITE+2
                    STA        SG_gsosCLOSE+2
*--
                    LDA        #$0001               ; EOF
                    STA        SG_gsosSETMARK+4
                    JSL        GSOS                 ; Set Mark to End
                    dw         $2016                ; Code for command 'SetMark'
                    adrl       SG_gsosSETMARK       ; Parameters Table Address   
                    BCS        SG_Close             ; Error
*--
                    LDA        #$0006               ; Data Length = 6 bytes
                    STA        SG_gsosWRITE+8
                    STZ        SG_gsosWRITE+10
                    JSL        GSOS                 ; Write Data to file
                    dw         $2013                ; Code for command 'Write'
                    adrl       SG_gsosWRITE         ; Parameters Table Address   
                    BCS        SG_Close             ; Error
*--                 
SG_Close            JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       SG_gsosCLOSE         ; Parameters Table Address 
*----
SG_End              RTS               

*----------   LoadGame/SaveGame Variables   ---------
SaveGameFilePath    strl       '1/Disk1/DragonsLair.sav'
SaveGameBuffer      HEX        0000,00,00,00,00     ; Scene ID, Score1, Life, Score3, Score2 
*----
SG_gsosDESTROY      dw         1                    ; Number of parameters
                    adrl       SaveGameFilePath     ; +2  File Pathname
*--                    
SG_gsosCREATE       dw         5                    ; Number of parameters
                    adrl       SaveGameFilePath     ; +2  File Pathname
                    ds         2                    ; +6  File Access
                    ds         2                    ; +8  File Type
                    ds         4                    ; +10 File AuxType       
                    ds         2                    ; +14 Storage Type
*--
SG_gsosOPEN         dw         3                    ; Number of parameters (Open in Write Mode)
                    ds         2                    ; +2 File Descriptor
                    adrl       SaveGameFilePath     ; +4 File Pathname
                    ds         2                    ; +8 Request Access (1=Read,2=Write)
*--
SG_gsosSETMARK      dw         3                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor
                    ds         2                    ; +4 Base
                    ds         4                    ; +6 Displacement
*--
SG_gsosWRITE        dw         5                    ; Number of parameters
                    ds         2                    ; +2  File Descriptor
                    adrl       SaveGameBuffer       ; +4  Data Buffer Address (init with an existing buffer path, so we only have to change 2 bytes)
                    ds         4                    ; +8  Data Size (Byte)
                    ds         4                    ; +12 Number of byte written to disk
                    ds         2                    ; +16 Cache Priority (#$0000 = No Cache)
*--
SG_gsosCLOSE        dw         1                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor
*----
LG_gsosOPEN         dw         2                    ; Number of parameters (Open in Read Mode)
                    ds         2                    ; +2 File Descriptor
                    adrl       SaveGameFilePath     ; +4 File Pathname
*--
LG_gsosGETEOF       dw         2                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor
                    ds         4                    ; +2 eof
*--
LG_gsosREAD         dw         4                    ; Number of parameters
                    ds         2                    ; +2  File Descriptor
                    ds         4                    ; +4  Data Buffer Address
                    ds         4                    ; +8  requestCount
                    ds         4                    ; +12 transferCount
*--                    
LG_gsosCLOSE        dw         1                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor


*************************************************************************
*  CheckSaveDisk :  Make sure Disk #1 is online (OUT:Carry=SetOnError). *
*************************************************************************
CheckSaveDisk       LDA        #CSD_FilePath_HDrive ; Test Hard Drive installation
                    JSR        IsFileExist
                    BNE        CSD_End
                    LDA        #CSD_FileData_Floppy ; Diskette installation
                    JSR        IsFileExist
                    BNE        CSD_End                    
*----
CSD_AskForFloppy    LDA        #$0001               ; A = Disk Number 1-8
                    JSR        InsertDiskScreen
                    LDA        #CSD_FileData_Floppy ; Diskette installation
                    JSR        IsFileExist
                    BNE        CSD_End
*--
                    LDX        #$001E               ; Display Error Message
CSD_InstallPalette  LDA        CSD_Palette,X
                    STAL       $E19E20,X
                    DEX
                    DEX
                    BPL        CSD_InstallPalette
*--
                    LDX        #$001E
                    LDA        #$0101               ; Palette #1
CSD_InstallSCB      STAL       $E19D54,X
                    DEX
                    DEX
                    BPL        CSD_InstallSCB
*--
                    LDX        #$13FE               ; Clear 32 lines
                    LDA        #$0000
CSD_Cleanup         STAL       $E15480,X
                    DEX
                    DEX
                    BPL        CSD_Cleanup
*--
                    LDX        #CSD_ErrorMessage_1  ; Can't access File
                    LDY        #$570D
                    JSR        DisplayString  
                    LDX        #CSD_ErrorMessage_2  ; Retry or Cancel
                    LDY        #$5FEA
                    JSR        DisplayString  
*--                    
CSD_WaitForKey      JSR        WaitForKey           ; Wait for a Key
                    CMP        #$0052               ; R
                    BEQ        CSD_Retry
                    CMP        #$0072               ; r
                    BEQ        CSD_Retry
                    CMP        #$0043               ; C
                    BEQ        CSD_Error_End
                    CMP        #$0063               ; c
                    BEQ        CSD_Error_End
                    BRA        CSD_WaitForKey 
*--
CSD_Retry           LDA        #CSD_FileData_Floppy ; Diskette installation
                    JSR        IsFileExist
                    BCC        CSD_End
                    BCS        CSD_WaitForKey
*----
CSD_Error_End       SEC                             ; Disk not available
                    RTS
*--                    
CSD_End             CLC                             ; Disk is available              
                    RTS

*----------   CheckSaveDisk Variables   ---------
CSD_FilePath_HDrive strl       '1/Disk1/Data001.BIN'
CSD_FileData_Floppy strl       '/DragonsLair1.D1/Disk1/Data001.bin'
CSD_ErrorMessage_1  ASC        'Can"t access File Disk1/Data001.bin',00
CSD_ErrorMessage_2  ASC        '[R]etry or [C]ancel',00
CSD_Palette         HEX        4A00,FF0F,0000,800F,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000


****************************************************************
*  SetSaveLevel() :  Ask for the Save at the end of the Scene. *
****************************************************************
SetSaveLevel        STA        SaveLevelStatus
                    RTS


*******************************************************************************
*  GetSaveLevel() :  Ask for the Save at the end of the Scene (OUT:A=Status). *
*******************************************************************************
GetSaveLevel        LDA        SaveLevelStatus
                    RTS

*----------   SetSaveLevel/GetSaveLevel Variables   ---------
SaveLevelStatus     HEX        0000


*------------------------ Other Source Files -----------------------------------

*--  Screen Functions
                    PUT        DragonsLair1.Screen  ; Screen Functions

*--  Shared Functions
                    PUT        DragonsLair1.Shared  ; Shared Functions
                    
*--  Disk Data : Scene 
                    PUT        DragonsLair1.Data    ; DiskX/DataXXX.bin information tables

*------------------------ 32 KB Multi-purpose Buffer ---------------------------

BUFFER32KB                                          ; 32 KB
BUFFERPIXEL         DS         32000                ; 320*200 4bpp
BUFFERSCB           DS         256                  ; 1 byte / line
BUFFERPALETTE       DS         512                  ; 16 palettes * 16 colors * 2 bytes / color
                
                    ASC        '- End of File -'

********************************************************************************
